@@ -20,15 +20,12 @@
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include <config.h>
 
 #include <string.h>
 
 #include <simgear/compiler.h>
 
-#include <boost/foreach.hpp>
 #include <string>
 
 #include <osg/ref_ptr>
@@ -62,16 +59,14 @@ using namespace simgear;
 class FGAIModelData : public simgear::SGModelData {
 public:
     FGAIModelData(SGPropertyNode *root = nullptr)
-        : _nasal( new FGNasalModelDataProxy(root) )
+      : _root (root)
     {
     }
 
 
-    ~FGAIModelData()
-    {
-    }
+    ~FGAIModelData() = default;
 
-    virtual FGAIModelData* clone() const { return new FGAIModelData(); }
+    FGAIModelData* clone() const override { return new FGAIModelData(); }
 
     /** osg callback, thread-safe */
     void modelLoaded(const std::string& path, SGPropertyNode *prop, osg::Node *n)
@@ -81,17 +76,23 @@ public:
             return;
 
         _modelLoaded[path] = true;
-        _ready = true;
 
         if(prop->hasChild("interior-path")){
             _interiorPath = prop->getStringValue("interior-path");
             _hasInteriorPath = true;
         }
 
+        // save radius of loaded model for updating LOD
+        auto bs = n->getBound();
+        if (bs.valid()) {
+            _radius = bs.radius();
+        }
+
         _fxpath = prop->getStringValue("sound/path");
-        _nasal->modelLoaded(path, prop, n);
-
-
+        _nasal[path] = std::unique_ptr<FGNasalModelDataProxy>(new FGNasalModelDataProxy(_root));
+        _nasal[path]->modelLoaded(path, prop, n);
+        _ready = true;
+        _initialized = false;
     }
 
     /** init hook to be called after model is loaded.
@@ -106,16 +107,19 @@ public:
     bool getInteriorLoaded(void) { return _interiorLoaded;}
     bool hasInteriorPath(void) { return _hasInteriorPath;}
     inline std::string& getInteriorPath() { return _interiorPath; }
+    inline float getRadius() { return _radius; }
 private:
-    std::unique_ptr<FGNasalModelDataProxy> _nasal;
     std::string _fxpath;
     std::string _interiorPath;
 
     std::map<string, bool> _modelLoaded;
+    std::map<string, std::unique_ptr<FGNasalModelDataProxy>> _nasal;
     bool _ready = false;
     bool _initialized = false;
     bool _hasInteriorPath = false;
     bool _interiorLoaded = false;
+    float _radius = -1.0;
+    SGPropertyNode* _root;
 };
 
 FGAIBase::FGAIBase(object_type ot, bool enableHot) :
@@ -240,7 +244,7 @@ void FGAIBase::readFromScenario(SGPropertyNode* scFileNode)
     setLatitude(scFileNode->getDoubleValue("latitude", 0.0));
     setBank(scFileNode->getDoubleValue("roll", 0.0));
     setPitch(scFileNode->getDoubleValue("pitch", 0.0));
-    
+
     SGPropertyNode* submodels = scFileNode->getChild("submodels");
 
     if (submodels) {
@@ -248,8 +252,11 @@ void FGAIBase::readFromScenario(SGPropertyNode* scFileNode)
         setSMPath(submodels->getStringValue("path", ""));
     }
 
+    // By default, scenarios include the full path to any aircraft.
+    _searchOrder = DATA_ONLY;
+
     string searchOrder = scFileNode->getStringValue("search-order", "");
-    if (!searchOrder.empty()) {           
+    if (!searchOrder.empty()) {
         if (searchOrder == "DATA_ONLY") {
             _searchOrder = DATA_ONLY;
         } else if (searchOrder == "PREFER_AI") {
@@ -258,9 +265,9 @@ void FGAIBase::readFromScenario(SGPropertyNode* scFileNode)
             _searchOrder = PREFER_DATA;
         } else
             SG_LOG(SG_AI, SG_WARN, "invalid model search order " << searchOrder << ". Use either DATA_ONLY, PREFER_AI or PREFER_DATA");
-        
+
     }
-    
+
     const string modelLowres = scFileNode->getStringValue("model-lowres", "");
     if (!modelLowres.empty()) {
         setPathLowres(modelLowres);
@@ -281,6 +288,43 @@ void FGAIBase::update(double dt) {
     ft_per_deg_lat = 366468.96 - 3717.12 * cos(pos.getLatitudeRad());
     ft_per_deg_lon = 365228.16 * cos(pos.getLatitudeRad());
 
+    if ((_modeldata)&&(_modeldata->needInitilization()))
+    {
+        // process deferred nasal initialization,
+        // which must be done in main thread
+        _modeldata->init();
+
+        // update LOD radius from loaded modeldata
+        auto radius = _modeldata->getRadius();
+        if (radius > 0) {
+            _model->setRadius(radius);
+            _model->dirtyBound();
+        }
+        else {
+            SG_LOG(SG_AI, SG_WARN, "AIBase: model radius not set.");
+        }
+
+        // sound initialization
+        if (fgGetBool("/sim/sound/aimodels/enabled",false))
+        {
+            const string& fxpath = _modeldata->get_sound_path();
+            if (fxpath != "")
+            {
+                props->setStringValue("sim/sound/path", fxpath.c_str());
+
+                // Remove any existing sound FX (e.g. from another model)
+                removeSoundFx();
+
+                // initialize the sound configuration
+                std::stringstream name;
+                name <<  "aifx:";
+                name << _refID;
+                _fx = new FGFX(name.str(), props);
+                _fx->init();
+            }
+        }
+    }
+
     if ( _fx )
     {
         // update model's audio sample values
@@ -294,29 +338,6 @@ void FGAIBase::update(double dt) {
                             pitch*speed );
         _fx->set_velocity( velocity );
     }
-    else if ((_modeldata)&&(_modeldata->needInitilization()))
-    {
-        // process deferred nasal initialization,
-        // which must be done in main thread
-        _modeldata->init();
-
-        // sound initialization
-        if (fgGetBool("/sim/sound/aimodels/enabled",false))
-        {
-            const string& fxpath = _modeldata->get_sound_path();
-            if (fxpath != "")
-            {
-                props->setStringValue("sim/sound/path", fxpath.c_str());
-
-                // initialize the sound configuration
-                std::stringstream name;
-                name <<  "aifx:";
-                name << _refID;
-                _fx = new FGFX(name.str(), props);
-                _fx->init();
-            }
-        }
-    }
 
     updateInterior();
 }
@@ -374,7 +395,7 @@ void FGAIBase::updateLOD()
                 _model->setRange(0, 0.0, FLT_MAX); // only one model.
             }
         }
-        else if ((int)maxRangeBare == (int)maxRangeDetail) 
+        else if ((int)maxRangeBare == (int)maxRangeDetail)
         {
             // low detail model  (only)
             if (_model->getNumFileNames() == 2) {
@@ -490,7 +511,7 @@ void FGAIBase::Transform() {
  */
 std::vector<std::string> FGAIBase::resolveModelPath(ModelSearchOrder searchOrder)
 {
-    std::vector<std::string> path_list;
+    string_list path_list;
 
     if (searchOrder == DATA_ONLY) {
         SG_LOG(SG_AI, SG_DEBUG, "Resolving model path:  DATA only");
@@ -500,7 +521,7 @@ std::vector<std::string> FGAIBase::resolveModelPath(ModelSearchOrder searchOrder
             _installed = true;
             SG_LOG(SG_AI, SG_DEBUG, "Found model " << p);
             path_list.push_back(p);
-            
+
             if (!model_path_lowres.empty()) {
                 p = simgear::SGModelLib::findDataFile(model_path_lowres);
                 if (!p.empty()) {
@@ -510,7 +531,8 @@ std::vector<std::string> FGAIBase::resolveModelPath(ModelSearchOrder searchOrder
             }
         } else {
             // No model, so fall back to the default
-            path_list.push_back(fgGetString("/sim/multiplay/default-model", default_model));
+            const SGPath defaultModelPath = SGPath::fromUtf8(fgGetString("/sim/multiplay/default-model", default_model));
+            path_list.push_back(defaultModelPath.utf8Str());
         }
     } else {
         SG_LOG(SG_AI, SG_DEBUG, "Resolving model path:  PREFER_AI/PREFER_DATA");
@@ -518,8 +540,8 @@ std::vector<std::string> FGAIBase::resolveModelPath(ModelSearchOrder searchOrder
         for (SGPath p : globals->get_data_paths("AI")) {
             p.append(model_path);
             if (p.exists()) {
-                SG_LOG(SG_AI, SG_DEBUG, "Found AI model: " << p.local8BitStr());
-                path_list.push_back(p.local8BitStr());
+                SG_LOG(SG_AI, SG_DEBUG, "Found AI model: " << p);
+                path_list.push_back(p.utf8Str());
                 break;
             }
         }
@@ -540,8 +562,8 @@ std::vector<std::string> FGAIBase::resolveModelPath(ModelSearchOrder searchOrder
             for (SGPath p : globals->get_data_paths()) {
                 p.append(fallback_path);
                 if (p.exists()) {
-                    SG_LOG(SG_AI, SG_DEBUG, "Found fallback model path for index " << _fallback_model_index << ": " << p.local8BitStr());
-                    path_list.push_back(p.local8BitStr());
+                    SG_LOG(SG_AI, SG_DEBUG, "Found fallback model path for index " << _fallback_model_index << ": " << p);
+                    path_list.push_back(p.utf8Str());
                     break;
                 }
             }
@@ -714,7 +736,7 @@ void FGAIBase::bind() {
     // FG 2018.4, so create an alias in case anyone is relying on the old name
     auto node = props->getNode("controls/flight/longitude-mode", true);
     node->alias(props->getNode("controls/flight/vertical-mode"));
-    
+
     props->setDoubleValue("controls/flight/target-alt", altitude_ft);
     props->setDoubleValue("controls/flight/target-pitch", pitch);
 
@@ -755,7 +777,7 @@ double FGAIBase::UpdateRadar(FGAIManager* manager)
     double d = dist(SGVec3d::fromGeod(pos), globals->get_aircraft_position_cart());
     double dFt = d * SG_METER_TO_FEET;
     in_range = (d < radar_range_m);
-    
+
     if (!force_on && !in_range) {
         return dFt * dFt;
     }
