@@ -10,6 +10,8 @@
 
 #include "submodel.hxx"
 
+#include <algorithm>
+
 #include <simgear/structure/exception.hxx>
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/math/sg_geodesy.hxx>
@@ -18,7 +20,6 @@
 #include <Main/fg_props.hxx>
 #include <Main/util.hxx>
 
-
 #include "AIBase.hxx"
 #include "AIManager.hxx"
 #include "AIBallistic.hxx"
@@ -49,7 +50,7 @@ FGSubmodelMgr::~FGSubmodelMgr()
 
 FGAIManager* FGSubmodelMgr::aiManager()
 {
-   return (FGAIManager*)globals->get_subsystem("ai-model");
+   return globals->get_subsystem<FGAIManager>();
 }
 
 void FGSubmodelMgr::init()
@@ -100,6 +101,12 @@ void FGSubmodelMgr::postinit()
     //_model_added_node->addChangeListener(this, false);
 }
 
+void FGSubmodelMgr::shutdown()
+{
+    std::for_each(submodels.begin(), submodels.end(), [](submodel* sm) { delete sm; });
+    submodels.clear();
+}
+
 void FGSubmodelMgr::bind()
 {
 }
@@ -398,8 +405,9 @@ void FGSubmodelMgr::transform(submodel *sm)
         _y_offset = sm->y_offset->get_value();
         _z_offset = sm->z_offset->get_value();
     }
-
-    setOffsetPos();
+    
+    // this updates the 'offsetpos' member of us
+    setOffsetPos(sm);
 
     // Compute initial orientation using yaw and pitch offsets and parent's orientation
     const double yaw_offset = sm->yaw_offset->get_value();
@@ -438,12 +446,6 @@ void FGSubmodelMgr::transform(submodel *sm)
     }
 }
 
-void FGSubmodelMgr::updatelat(double lat)
-{
-    ft_per_deg_latitude = 366468.96 - 3717.12 * cos(lat / SG_RADIANS_TO_DEGREES);
-    ft_per_deg_longitude = 365228.16 * cos(lat / SG_RADIANS_TO_DEGREES);
-}
-
 void FGSubmodelMgr::loadAI()
 {
     SG_LOG(SG_AI, SG_DEBUG, "Submodels: Loading AI submodels");
@@ -695,18 +697,22 @@ void FGSubmodelMgr::loadSubmodels()
     subsubmodels.clear();
 }
 
-SGVec3d FGSubmodelMgr::getCartOffsetPos() const
+SGVec3d FGSubmodelMgr::getCartOffsetPos(submodel* sm) const
 {
-    // Convert geodetic positions to geocentered
-    SGVec3d cartuserPos = SGVec3d::fromGeod(userpos);
-
     // Transform to the right coordinate frame, configuration is done in
     // the x-forward, y-right, z-up coordinates (feet), computation
     // in the simulation usual body x-forward, y-right, z-down coordinates
     // (meters) )
-    SGVec3d _off(_x_offset * SG_FEET_TO_METER,
-        _y_offset * SG_FEET_TO_METER,
-        -_z_offset * SG_FEET_TO_METER);
+    SGVec3d offset;
+    if (sm->offsets_in_meter) {
+        offset = SGVec3d(-sm->x_offset->get_value(),
+                         sm->y_offset->get_value(),
+                         -sm->z_offset->get_value());
+    } else {
+        offset = SGVec3d(sm->x_offset->get_value() * SG_FEET_TO_METER,
+                         sm->y_offset->get_value() * SG_FEET_TO_METER,
+                         -sm->z_offset->get_value() * SG_FEET_TO_METER);
+    }
 
     // Transform the user position to the horizontal local coordinate system.
     SGQuatd hlTrans = SGQuatd::fromLonLat(userpos);
@@ -718,19 +724,23 @@ SGVec3d FGSubmodelMgr::getCartOffsetPos() const
        IC.elevation,
        IC.roll);
 
+    // postrotate by any pitch/heasing/roll offset
+    hlTrans *= SGQuatd::fromYawPitchRollDeg(sm->yaw_offset->get_value(),
+                                            sm->pitch_offset->get_value(),
+                                            0.0);
+    
     // The offset converted to the usual body fixed coordinate system
     // rotated to the earth-fixed coordinates axis
-    SGVec3d off = hlTrans.backTransform(_off);
+    SGVec3d off = hlTrans.backTransform(offset);
 
     // Add the position offset of the user model to get the geocentered position
-    SGVec3d offsetPos = cartuserPos + off;
-    return offsetPos;
+    return SGVec3d::fromGeod(userpos) + off;
 }
 
-void FGSubmodelMgr::setOffsetPos()
+void FGSubmodelMgr::setOffsetPos(submodel* sm)
 {
     // Convert the offset geocentered position to geodetic
-    SGVec3d cartoffsetPos = getCartOffsetPos();
+    SGVec3d cartoffsetPos = getCartOffsetPos(sm);
     SGGeodesy::SGCartToGeod(cartoffsetPos, offsetpos);
 }
 
@@ -807,3 +817,8 @@ void FGSubmodelMgr::setParentNode(int id)
         SG_LOG(SG_AI, SG_ALERT, "AISubmodel: parent node not found ");
     }
 }
+
+
+// Register the subsystem.
+SGSubsystemMgr::Registrant<FGSubmodelMgr> registrantFGSubmodelMgr(
+    SGSubsystemMgr::POST_FDM);
