@@ -155,9 +155,13 @@ public:
     BasicRealWxController( SGPropertyNode_ptr rootNode, MetarRequester * metarRequester );
     virtual ~BasicRealWxController ();
 
-    virtual void init ();
-    virtual void reinit ();
-    virtual void shutdown ();
+    // Subsystem API.
+    void bind() override;
+    void init() override;
+    void reinit() override;
+    void shutdown() override;
+    void unbind() override;
+    void update(double dt) override;
     
     /**
      * Create a metar-property binding at the specified property path,
@@ -171,11 +175,8 @@ public:
 
     typedef std::vector<LiveMetarProperties_ptr> MetarPropertiesList;
     MetarPropertiesList::iterator findMetarAtPath(const string &propPath);
-protected:
-    void bind();
-    void unbind();
-    void update( double dt );
 
+protected:
     void checkNearbyMetar();
 
     long getMetarMaxAgeMin() const { return _max_age_n == NULL ? 0 : _max_age_n->getLongValue(); }
@@ -189,7 +190,6 @@ protected:
     simgear::TiedPropertyList _tiedProperties;
     MetarPropertiesList _metarProperties;
     MetarRequester* _requester;
-
 };
 
 static bool commandRequestMetar(const SGPropertyNode * arg, SGPropertyNode * root)
@@ -274,7 +274,7 @@ void BasicRealWxController::init()
     update(0); // fetch data ASAP
     
     globals->get_event_mgr()->addTask("checkNearbyMetar", this,
-                                      &BasicRealWxController::checkNearbyMetar, 60 );
+                                      &BasicRealWxController::checkNearbyMetar, 10 );
 }
 
 void BasicRealWxController::reinit()
@@ -395,12 +395,13 @@ void BasicRealWxController::checkNearbyMetar()
     catch( sg_exception & ) {
       return;
     }
-    
 }
 
+
 /* -------------------------------------------------------------------------------- */
 
-class NoaaMetarRealWxController : public BasicRealWxController, MetarRequester {
+class NoaaMetarRealWxController : public BasicRealWxController, MetarRequester
+{
 public:
     NoaaMetarRealWxController( SGPropertyNode_ptr rootNode );
 
@@ -410,13 +411,24 @@ public:
     virtual ~NoaaMetarRealWxController()
     {
     }
+
+    // Subsystem identification.
+    static const char* staticSubsystemClassId() { return "noaa-metar-real-wx-controller"; }
+
 private:
-    
+    std::string noaa_base_url;
 };
 
 NoaaMetarRealWxController::NoaaMetarRealWxController( SGPropertyNode_ptr rootNode ) :
   BasicRealWxController(rootNode, this )
 {
+    // default to hardcoded URL for compatibility
+    noaa_base_url = "https://tgftp.nws.noaa.gov/data/observations/metar/stations/[station].TXT";
+
+    // override with environment/realwx/metar-url (if present)
+    SGPropertyNode *urlNode = _rootNode->getNode("metar-url", false);
+    if (urlNode != nullptr)
+        noaa_base_url = urlNode->getStringValue();
 }
 
 void NoaaMetarRealWxController::requestMetar
@@ -425,15 +437,14 @@ void NoaaMetarRealWxController::requestMetar
   const std::string& id
 )
 {
-  static const std::string NOAA_BASE_URL =
-    "https://tgftp.nws.noaa.gov/data/observations/metar/stations/";
   class NoaaMetarGetRequest:
     public simgear::HTTP::MemoryRequest
   {
     public:
       NoaaMetarGetRequest( LiveMetarProperties_ptr metarDataHandler,
-                           const std::string& stationId ):
-        MemoryRequest(NOAA_BASE_URL + stationId + ".TXT"),
+                           const std::string& stationId, 
+                           const std::string &base_url):
+        MemoryRequest( simgear::strutils::replace(base_url, "[station]",stationId) ),
         _metarDataHandler(metarDataHandler)
       {
         std::ostringstream buf;
@@ -482,10 +493,20 @@ void NoaaMetarRealWxController::requestMetar
   );
   FGHTTPClient* http = globals->get_subsystem<FGHTTPClient>();
   if (http) {
-      http->makeRequest(new NoaaMetarGetRequest(metarDataHandler, upperId));
+      http->makeRequest(new NoaaMetarGetRequest(metarDataHandler, upperId, noaa_base_url));
   }
 }
 
+// Register the subsystem.
+#if 0
+SGSubsystemMgr::Registrant<NoaaMetarRealWxController> registrantNoaaMetarRealWxController(
+    SGSubsystemMgr::GENERAL,
+    {{"environment", SGSubsystemMgr::Dependency::SOFT},
+     {"FGHTTPClient", SGSubsystemMgr::Dependency::SOFT},
+     {"realwx", SGSubsystemMgr::Dependency::SOFT}});
+#endif
+
+
 /* -------------------------------------------------------------------------------- */
     
 RealWxController * RealWxController::createInstance( SGPropertyNode_ptr rootNode )
