@@ -45,6 +45,7 @@
 #include "QmlPositioned.hxx"
 #include "PixmapImageItem.hxx"
 #include "AirportDiagram.hxx"
+#include "CarrierDiagram.hxx"
 #include "NavaidDiagram.hxx"
 #include "RouteDiagram.hxx"
 #include "QmlRadioButtonHelper.hxx"
@@ -52,6 +53,9 @@
 #include "NavaidSearchModel.hxx"
 #include "FlightPlanController.hxx"
 #include "ModelDataExtractor.hxx"
+#include "CarriersLocationModel.hxx"
+#include "SetupRootDialog.hxx"
+#include "HoverArea.hxx"
 
 using namespace simgear::pkg;
 
@@ -90,6 +94,9 @@ LauncherController::LauncherController(QObject *parent, QWindow* window) :
 
     m_aircraftSearchModel = new AircraftProxyModel(this, m_aircraftModel);
 
+    m_favouriteAircraftModel = new AircraftProxyModel(this, m_aircraftModel);
+    m_favouriteAircraftModel->setShowFavourites(true);
+
     m_aircraftHistory = new RecentAircraftModel(m_aircraftModel, this);
 
     connect(m_aircraftModel, &AircraftItemModel::aircraftInstallCompleted,
@@ -102,12 +109,12 @@ LauncherController::LauncherController(QObject *parent, QWindow* window) :
             this, &LauncherController::updateSelectedAircraft);
 
     QSettings settings;
-    LocalAircraftCache::instance()->setPaths(settings.value("aircraft-paths").toStringList());
-    LocalAircraftCache::instance()->scanDirs();
     m_aircraftModel->setPackageRoot(globals->packageRoot());
 
+    m_aircraftGridMode = settings.value("aircraftGridMode").toBool();
+
     m_subsystemIdleTimer = new QTimer(this);
-    m_subsystemIdleTimer->setInterval(10);
+    m_subsystemIdleTimer->setInterval(5);
     connect(m_subsystemIdleTimer, &QTimer::timeout, []()
        {globals->get_subsystem_mgr()->update(0.0);});
     m_subsystemIdleTimer->start();
@@ -142,6 +149,7 @@ void LauncherController::initQML()
     qmlRegisterUncreatableType<MPServersModel>("FlightGear.Launcher", 1, 0, "MPServers", "Singleton API");
 
     qmlRegisterType<NavaidSearchModel>("FlightGear", 1, 0, "NavaidSearch");
+    qmlRegisterType<CarriersLocationModel>("FlightGear", 1, 0, "CarriersModel");
 
     qmlRegisterUncreatableType<Units>("FlightGear", 1, 0, "Units", "Only for enum");
     qmlRegisterType<UnitsModel>("FlightGear", 1, 0, "UnitsModel");
@@ -161,9 +169,11 @@ void LauncherController::initQML()
 
     qmlRegisterType<PixmapImageItem>("FlightGear", 1, 0, "PixmapImage");
     qmlRegisterType<AirportDiagram>("FlightGear", 1, 0, "AirportDiagram");
+    qmlRegisterType<CarrierDiagram>("FlightGear", 1, 0, "CarrierDiagram");
     qmlRegisterType<NavaidDiagram>("FlightGear", 1, 0, "NavaidDiagram");
     qmlRegisterType<RouteDiagram>("FlightGear", 1, 0, "RouteDiagram");
     qmlRegisterType<QmlRadioButtonGroup>("FlightGear", 1, 0, "RadioButtonGroup");
+    qmlRegisterType<HoverArea>("FlightGear", 1, 0, "HoverArea");
 
     qmlRegisterType<ModelDataExtractor>("FlightGear", 1, 0, "ModelDataExtractor");
 
@@ -183,6 +193,7 @@ void LauncherController::setInAppMode()
 	m_inAppMode = true;
 	m_keepRunningInAppMode = true;
 	m_appModeResult = true;
+    emit inAppChanged();
 }
 
 bool LauncherController::keepRunningInAppMode() const
@@ -278,17 +289,6 @@ void LauncherController::collectAircraftArgs()
             m_config->setArg("state", state);
         }
     }
-
-    // scenery paths
-    QSettings settings;
-    Q_FOREACH(QString path, settings.value("scenery-paths").toStringList()) {
-        m_config->setArg("fg-scenery", path);
-    }
-
-    // aircraft paths
-    Q_FOREACH(QString path, settings.value("aircraft-paths").toStringList()) {
-        m_config->setArg("fg-aircraft", path);
-    }
 }
 
 void LauncherController::saveAircraft()
@@ -391,6 +391,10 @@ QString LauncherController::selectAircraftStateAutomatically()
         }
     }
 
+    if (m_location->isCarrier() && m_location->isAirborneLocation() && m_selectedAircraftInfo->hasState("carrier-approach")) {
+        return "carrier-approach";
+    }
+
     if (m_location->isAirborneLocation() && m_selectedAircraftInfo->hasState("approach")) {
         return "approach";
     }
@@ -402,10 +406,14 @@ QString LauncherController::selectAircraftStateAutomatically()
         if (m_selectedAircraftInfo->hasState("parking")) {
             return "parking";
         }
-    } else {
-        // also try 'engines-running'?
-        if (m_selectedAircraftInfo->hasState("take-off"))
-            return "take-off";
+    }
+
+    if (m_location->isCarrier() && m_selectedAircraftInfo->hasState("carrier-take-off")) {
+        return "carrier-take-off";
+    }
+
+    if (m_selectedAircraftInfo->hasState("take-off")) {
+        return "take-off";
     }
 
     return {}; // failed to compute, give up
@@ -622,10 +630,6 @@ bool LauncherController::validateMetarString(QString metar)
 
 void LauncherController::requestInstallUpdate(QUrl aircraftUri)
 {
-    // also select, otherwise UI is confusing
-    m_selectedAircraft = aircraftUri;
-    updateSelectedAircraft();
-
     simgear::pkg::PackageRef pref = packageForAircraftURI(aircraftUri);
     if (pref) {
         if (pref->isInstalled()) {
@@ -710,6 +714,20 @@ QVariantList LauncherController::defaultSplashUrls() const
     return urls;
 }
 
+QVariant LauncherController::loadUISetting(QString name, QVariant defaultValue) const
+{
+    QSettings settings;
+    if (!settings.contains(name))
+        return defaultValue;
+    return settings.value(name);
+}
+
+void LauncherController::saveUISetting(QString name, QVariant value) const
+{
+    QSettings settings;
+    settings.setValue(name, value);
+}
+
 void LauncherController::onAircraftInstalledCompleted(QModelIndex index)
 {
     maybeUpdateSelectedAircraft(index);
@@ -765,7 +783,7 @@ void LauncherController::requestChangeDataPath()
 {
 	QString currentLocText;
 	QSettings settings;
-	QString root = settings.value("fg-root").toString();
+	QString root = settings.value(SetupRootDialog::rootPathKey()).toString();
 	if (root.isNull()) {
 		currentLocText = tr("Currently the built-in data files are being used");
 	}
@@ -789,12 +807,7 @@ void LauncherController::requestChangeDataPath()
 		return;
 	}
 
-	{
-		QSettings settings;
-		// set the option to the magic marker value
-		settings.setValue("fg-root", "!ask");
-	} // scope the ensure settings are written nicely
-
+    SetupRootDialog::askRootOnNextLaunch();
     flightgear::restartTheApp();
 }
 
@@ -821,13 +834,24 @@ void LauncherController::saveConfigAs()
     m_config->saveConfigToFile(file);
 }
 
+void LauncherController::setAircraftGridMode(bool aircraftGridMode)
+{
+    if (m_aircraftGridMode == aircraftGridMode)
+        return;
+
+    QSettings settings;
+    settings.setValue("aircraftGridMode", aircraftGridMode);
+    m_aircraftGridMode = aircraftGridMode;
+    emit aircraftGridModeChanged(m_aircraftGridMode);
+}
+
 void LauncherController::setMinWindowSize(QSize sz)
 {
-	if (sz == m_minWindowSize)
-		return;
+    if (sz == m_minWindowSize)
+        return;
 
-	m_window->setMinimumSize(sz);
-	emit minWindowSizeChanged();
+    m_window->setMinimumSize(sz);
+    emit minWindowSizeChanged();
 }
 
 QUrl LauncherController::flyIconUrl() const
