@@ -52,6 +52,7 @@
 #include <simgear/structure/exception.hxx>
 #include <simgear/structure/subsystem_mgr.hxx>
 #include <simgear/misc/sg_path.hxx>
+#include <simgear/package/Root.hxx>
 #include <simgear/package/Catalog.hxx>
 #include <simgear/package/Package.hxx>
 #include <simgear/package/Install.hxx>
@@ -74,6 +75,7 @@
 #include "LauncherMainWindow.hxx"
 #include "LaunchConfig.hxx"
 #include "UnitsModel.hxx"
+#include "PathListModel.hxx"
 
 using namespace flightgear;
 using namespace simgear::pkg;
@@ -81,9 +83,24 @@ using std::string;
 
 namespace { // anonymous namespace
 
+struct ProgressLabel {
+    NavDataCache::RebuildPhase phase;
+    const char* label;
+};
+
+static std::initializer_list<ProgressLabel> progressStrings = {
+    {NavDataCache::REBUILD_READING_APT_DAT_FILES, QT_TRANSLATE_NOOP("initNavCache","Reading airport data")},
+    {NavDataCache::REBUILD_LOADING_AIRPORTS, QT_TRANSLATE_NOOP("initNavCache","Loading airports")},
+    {NavDataCache::REBUILD_FIXES, QT_TRANSLATE_NOOP("initNavCache","Loading waypoint data")},
+    {NavDataCache::REBUILD_NAVAIDS, QT_TRANSLATE_NOOP("initNavCache","Loading navigation data")},
+    {NavDataCache::REBUILD_POIS, QT_TRANSLATE_NOOP("initNavCache","Loading point-of-interest data")}
+};
+
 void initNavCache()
 {
-    QString baseLabel = QT_TRANSLATE_NOOP("initNavCache", "Initialising navigation data, this may take several minutes");
+    const char* baseLabelKey = QT_TRANSLATE_NOOP("initNavCache", "Initialising navigation data, this may take several minutes");
+    QString baseLabel= qApp->translate("initNavCache", baseLabelKey);
+
     NavDataCache* cache = NavDataCache::createInstance();
     if (cache->isRebuildRequired()) {
         QProgressDialog rebuildProgress(baseLabel,
@@ -95,51 +112,41 @@ void initNavCache()
                                            | Qt::WindowSystemMenuHint
                                            | Qt::MSWindowsFixedSizeDialogHint);
         rebuildProgress.setWindowModality(Qt::WindowModal);
+        rebuildProgress.setMinimumWidth(600);
+        rebuildProgress.setAutoReset(false);
+        rebuildProgress.setAutoClose(false);
         rebuildProgress.show();
-
-        NavDataCache::RebuildPhase phase = cache->rebuild();
-
-        while (phase != NavDataCache::REBUILD_DONE) {
-            // sleep to give the rebuild thread more time
-            SGTimeStamp::sleepForMSec(50);
-            phase = cache->rebuild();
-
-            switch (phase) {
-            case NavDataCache::REBUILD_READING_APT_DAT_FILES:
-                rebuildProgress.setLabelText(QT_TRANSLATE_NOOP("initNavCache","Reading airport data"));
-                break;
-
-            case NavDataCache::REBUILD_LOADING_AIRPORTS:
-                rebuildProgress.setLabelText(QT_TRANSLATE_NOOP("initNavCache","Loading airports"));
-                break;
-
-            case NavDataCache::REBUILD_FIXES:
-                rebuildProgress.setLabelText(QT_TRANSLATE_NOOP("initNavCache","Loading waypoint data"));
-                break;
-
-            case NavDataCache::REBUILD_NAVAIDS:
-                rebuildProgress.setLabelText(QT_TRANSLATE_NOOP("initNavCache","Loading navigation data"));
-                break;
-
-
-            case NavDataCache::REBUILD_POIS:
-                rebuildProgress.setLabelText(QT_TRANSLATE_NOOP("initNavCache","Loading point-of-interest data"));
-                break;
-
-            default:
+    
+        QTimer updateTimer;
+        updateTimer.setInterval(100);
+        QObject::connect(&updateTimer, &QTimer::timeout, [&cache, &rebuildProgress, &baseLabel]() {
+            auto phase = cache->rebuild();
+            if (phase == NavDataCache::REBUILD_DONE) {
+                rebuildProgress.done(0);
+                return;
+            }
+            
+            auto it = std::find_if(progressStrings.begin(), progressStrings.end(), [phase]
+                                   (const ProgressLabel& l) { return l.phase == phase; });
+            if (it == progressStrings.end()) {
                 rebuildProgress.setLabelText(baseLabel);
+            } else {
+                QString trans = qApp->translate("initNavCache", it->label);
+                rebuildProgress.setLabelText(trans);
             }
 
             if (phase == NavDataCache::REBUILD_UNKNOWN) {
                 rebuildProgress.setValue(0);
                 rebuildProgress.setMaximum(0);
             } else {
-                rebuildProgress.setValue(cache->rebuildPhaseCompletionPercentage());
+                rebuildProgress.setValue(static_cast<int>(cache->rebuildPhaseCompletionPercentage()));
                 rebuildProgress.setMaximum(100);
             }
-
-            QCoreApplication::processEvents();
-        }
+        });
+        
+        updateTimer.start(); // timer won't actually run until we process events
+        rebuildProgress.exec();
+        updateTimer.stop();
     }
 }
 
@@ -283,13 +290,15 @@ void initApp(int& argc, char** argv, bool doInitQSettings)
           QStringLiteral("org.flightgear.FlightGear.desktop"));
 #endif
         QTranslator* fallbackTranslator = new QTranslator(static_qApp.get());
-        if (!fallbackTranslator->load(QLatin1String(":/FlightGear_en.qm"))) {
+        if (!fallbackTranslator->load(QLatin1String(":/FlightGear_en_US.qm"))) {
             qWarning() << "Failed to load default (en) translations";
             delete fallbackTranslator;
         } else {
             static_qApp->installTranslator(fallbackTranslator);
         }
 
+        qInfo() << "UI languages:" << QLocale().uiLanguages();
+        
         QTranslator* translator = new QTranslator(static_qApp.get());
         // check for --langauge=xx option and prefer that over QLocale
         // detection of the locale if it exists
@@ -352,19 +361,22 @@ void initQSettings()
 bool checkKeyboardModifiersForSettingFGRoot()
 {
     initQSettings();
-
+#if defined(Q_OS_WIN)
+    const auto altState = GetAsyncKeyState(VK_MENU);
+    const auto shiftState = GetAsyncKeyState(VK_SHIFT);
+    if ((altState < 0) || (shiftState < 0))
+#else
     Qt::KeyboardModifiers mods = qApp->queryKeyboardModifiers();
-    if (mods & (Qt::AltModifier | Qt::ShiftModifier)) {
+    if (mods & (Qt::AltModifier | Qt::ShiftModifier))
+#endif
+    {
         qWarning() << "Alt/shift pressed during launch";
-        QSettings settings;
-        settings.setValue("fg-root", "!ask");
         return true;
     }
 
     return false;
 }
 
-
 void restartTheApp()
 {
     QStringList fgArgs;
@@ -401,7 +413,7 @@ void launcherSetSceneryPaths()
 // positions
     QSettings settings;
     // append explicit scenery paths
-    Q_FOREACH(QString path, settings.value("scenery-paths").toStringList()) {
+    Q_FOREACH(QString path, PathListModel::readEnabledPaths("scenery-paths-v2")) {
         globals->append_fg_scenery(path.toStdString());
     }
 
@@ -449,6 +461,15 @@ bool runLauncherDialog()
 
     fgInitPackageRoot();
 
+    // setup package language
+    auto lang = options->valueForOption("language");
+    if (!lang.empty()) {
+        globals->packageRoot()->setLocale(lang);
+    } else {
+        const auto langName = QLocale::languageToString(QLocale{}.language());
+        globals->packageRoot()->setLocale(langName.toStdString());
+    }
+
     // startup the HTTP system now since packages needs it
     FGHTTPClient* http = globals->add_new_subsystem<FGHTTPClient>();
 
@@ -464,7 +485,11 @@ bool runLauncherDialog()
     flightgear::WindowBuilder::setPoseAsStandaloneApp(false);
 
     LauncherMainWindow dlg;
-    dlg.show();
+    if (options->isOptionSet("enable-fullscreen")) {
+        dlg.showFullScreen();
+    } else {
+        dlg.show();
+    }
 
     int appResult = qApp->exec();
     if (appResult <= 0) {
