@@ -433,6 +433,9 @@ NewGUI::setStyle (void)
     
     SGPropertyNode *selected_style = globals->get_props()->getNode("sim/gui/selected-style", true);
 
+    // n->copy() doesn't delete existing nodes, so need to clear them all
+    // first.
+    selected_style->removeAllChildren();
     n->copy(selected_style);
 
     //if (selected_style && n)
@@ -459,4 +462,9 @@ NewGUI::setupFont (SGPropertyNode *node)
     return;
 }
 
+
+// Register the subsystem.
+SGSubsystemMgr::Registrant<NewGUI> registrantNewGUI(
+    SGSubsystemMgr::INIT);
+
 // end of new_gui.cxx
