@@ -30,11 +30,12 @@
 #  include <windows.h>
 #endif
 
+#include <cmath>
+
 #include <simgear/props/props_io.hxx>
 #include "FGDeviceConfigurationMap.hxx"
 #include <Main/fg_props.hxx>
 #include <Scripting/NasalSys.hxx>
-#include <boost/foreach.hpp>
 
 using simgear::PropertyList;
 
@@ -100,7 +101,7 @@ void FGJoystickInput::_remove(bool all)
         // do not remove predefined joysticks info on reinit
         if (all || (!joy->predefined))
             js_nodes->removeChild("js", i);
-        
+
         joy->plibJS.reset();
         joy->clearAxesAndButtons();
     }
@@ -118,7 +119,8 @@ void FGJoystickInput::init()
   for (int i = 0; i < MAX_JOYSTICKS; i++) {
     jsJoystick * js = new jsJoystick(i);
     joysticks[i].plibJS.reset(js);
-    
+    joysticks[i].initializing = true;
+
     if (js->notWorking()) {
       SG_LOG(SG_INPUT, SG_DEBUG, "Joystick " << i << " not found");
       continue;
@@ -258,7 +260,7 @@ void FGJoystickInput::postinit()
               n_axis = num_node->getIntValue("unix", -1);
           }
         #endif
-          
+
           // Silently ignore platforms that are not specified within the
           // <number></number> section
           if (n_axis < 0) {
@@ -297,7 +299,7 @@ void FGJoystickInput::postinit()
     // Initialize the buttons.
     //
     PropertyList buttons = js_node->getChildren("button");
-    BOOST_FOREACH( SGPropertyNode * button_node, buttons ) {
+    for (auto button_node : buttons ) {
       size_t n_but = button_node->getIndex();
 
       const SGPropertyNode * num_node = button_node->getChild("number");
@@ -334,58 +336,96 @@ void FGJoystickInput::updateJoystick(int index, FGJoystickInput::joystick* joy,
   int modifiers = fgGetKeyModifiers();
   int buttons;
   bool pressed, last_state;
+  bool axes_initialized;
   float delay;
-  
+
   jsJoystick * js = joy->plibJS.get();
-  if (js == 0 || js->notWorking())
+  if (js == 0 || js->notWorking()) {
+    joysticks[index].initializing = true;
+    if (js) {
+      joysticks[index].init_dt += dt;
+      if (joysticks[index].init_dt >= 1.0) {
+        joysticks[index].plibJS.reset( new jsJoystick(index) );
+        joysticks[index].init_dt = 0.0;
+      }
+    }
     return;
-  
+  }
+
   js->read(&buttons, axis_values);
-  if (js->notWorking()) // If js is disconnected
+  if (js->notWorking()) { // If js is disconnected
+    joysticks[index].initializing = true;
     return;
-  
+  }
+
+  // Joystick axes can get initialized to extreme values, at least on Linux.
+  // Wait until one of the axes get a different value before continuing.
+  // https://sourceforge.net/p/flightgear/codetickets/2185/
+  axes_initialized = true;
+  if (joysticks[index].initializing) {
+
+    if (!joysticks[index].initialized) {
+      js->read(NULL, joysticks[index].values);
+      joysticks[index].initialized = true;
+    }
+
+    int j;
+    for (j = 0; j < joy->naxes; j++) {
+      if (axis_values[j] != joysticks[index].values[j]) break;
+    }
+    if (j < joy->naxes) {
+      joysticks[index].initializing = false;
+    } else {
+      axes_initialized = false;
+    }
+  }
+
   // Update device status
   SGPropertyNode_ptr status = status_node->getChild("joystick", index, true);
-  for (int j = 0; j < MAX_JOYSTICK_AXES; j++) {
-    status->getChild("axis", j, true)->setFloatValue(axis_values[j]);
+  if (axes_initialized) {
+    for (int j = 0; j < MAX_JOYSTICK_AXES; j++) {
+      status->getChild("axis", j, true)->setFloatValue(axis_values[j]);
+    }
   }
-  
+
   for (int j = 0; j < MAX_JOYSTICK_BUTTONS; j++) {
     status->getChild("button", j, true)->setBoolValue((buttons & (1u << j)) > 0 );
   }
-  
+
   // Fire bindings for the axes.
-  for (int j = 0; j < joy->naxes; j++) {
-    axis &a = joy->axes[j];
-    
-    // Do nothing if the axis position
-    // is unchanged; only a change in
-    // position fires the bindings.
-    // But only if there are bindings
-    if (fabs(axis_values[j] - a.last_value) > a.tolerance
-        && a.bindings[KEYMOD_NONE].size() > 0 ) {
-      a.last_value = axis_values[j];
-      for (unsigned int k = 0; k < a.bindings[KEYMOD_NONE].size(); k++)
-        a.bindings[KEYMOD_NONE][k]->fire(axis_values[j]);
-    }
-    
-    // do we have to emulate axis buttons?
-    last_state = joy->axes[j].low.last_state || joy->axes[j].high.last_state;
-    pressed = axis_values[j] < a.low_threshold || axis_values[j] > a.high_threshold;
-    delay = (pressed ? last_state ? a.interval_sec : a.delay_sec : a.release_delay_sec );
-    if(pressed || last_state) a.last_dt += dt;
-    else a.last_dt = 0;
-    if(a.last_dt >= delay) {
-      if (a.low.bindings[modifiers].size())
-        joy->axes[j].low.update( modifiers, axis_values[j] < a.low_threshold );
-      
-      if (a.high.bindings[modifiers].size())
-        joy->axes[j].high.update( modifiers, axis_values[j] > a.high_threshold );
-      
-      a.last_dt -= delay;
-    }
-  } // of axes iteration
-  
+  if (axes_initialized) {
+    for (int j = 0; j < joy->naxes; j++) {
+      axis &a = joy->axes[j];
+
+      // Do nothing if the axis position
+      // is unchanged; only a change in
+      // position fires the bindings.
+      // But only if there are bindings
+      if (fabs(axis_values[j] - a.last_value) > a.tolerance
+          && a.bindings[KEYMOD_NONE].size() > 0 ) {
+        a.last_value = axis_values[j];
+        for (unsigned int k = 0; k < a.bindings[KEYMOD_NONE].size(); k++)
+          a.bindings[KEYMOD_NONE][k]->fire(axis_values[j]);
+      }
+
+      // do we have to emulate axis buttons?
+      last_state = joy->axes[j].low.last_state || joy->axes[j].high.last_state;
+      pressed = axis_values[j] < a.low_threshold || axis_values[j] > a.high_threshold;
+      delay = (pressed ? last_state ? a.interval_sec : a.delay_sec : a.release_delay_sec );
+      if(pressed || last_state) a.last_dt += dt;
+      else a.last_dt = 0;
+      if(a.last_dt >= delay) {
+        if (a.low.bindings[modifiers].size())
+          joy->axes[j].low.update( modifiers, axis_values[j] < a.low_threshold );
+
+        if (a.high.bindings[modifiers].size())
+          joy->axes[j].high.update( modifiers, axis_values[j] > a.high_threshold );
+
+        a.last_dt -= delay;
+      }
+    } // of axes iteration
+  } // axes_initialized
+
   // Fire bindings for the buttons.
   for (int j = 0; j < joy->nbuttons; j++) {
     FGButton &b = joy->buttons[j];
@@ -409,3 +449,8 @@ void FGJoystickInput::update( double dt )
   }
 }
 
+
+// Register the subsystem.
+SGSubsystemMgr::Registrant<FGJoystickInput> registrantFGJoystickInput(
+    SGSubsystemMgr::GENERAL,
+    {{"nasal", SGSubsystemMgr::Dependency::HARD}});
