@@ -49,7 +49,6 @@
 #include "tcas.hxx"
 
 FGInstrumentMgr::FGInstrumentMgr () :
-  SGSubsystemGroup("FGInstrumentMgr"),
   _explicitGps(false)
 {    
 }
@@ -227,4 +226,9 @@ bool FGInstrumentMgr::build (SGPropertyNode* config_props)
     return true;
 }
 
+
+// Register the subsystem.
+SGSubsystemMgr::Registrant<FGInstrumentMgr> registrantFGInstrumentMgr(
+    SGSubsystemMgr::FDM);
+
 // end of instrument_manager.cxx
