@@ -103,7 +103,6 @@ static std::unique_ptr<SGInterpTable> static_terminalRangeInterp,
 
 // Constructor
 FGNavRadio::FGNavRadio(SGPropertyNode *node) :
-    _operable(false),
     play_count(0),
     _nav_search(true),
     _last_freq(0.0),
@@ -116,8 +115,6 @@ FGNavRadio::FGNavRadio(SGPropertyNode *node) :
     last_xtrack_error(0.0),
     xrate_ms(0.0),
     _localizerWidth(5.0),
-    _name(node->getStringValue("name", "nav")),
-    _num(node->getIntValue("number", 0)),
     _time_before_search_sec(-1.0),
     _gsCart(SGVec3d::zeros()),
     _gsAxis(SGVec3d::zeros()),
@@ -130,6 +127,8 @@ FGNavRadio::FGNavRadio(SGPropertyNode *node) :
     _gsNeedleDeflectionNorm(0.0),
     _audioIdent(NULL)
 {
+    readConfig(node, "nav");
+    
     if (!static_terminalRangeInterp.get()) {
     // one-time interpolator init
       SGPath path( globals->get_fg_root() );
@@ -145,8 +144,8 @@ FGNavRadio::FGNavRadio(SGPropertyNode *node) :
       static_highRangeInterp.reset(new SGInterpTable(high));
     }
   
-    string branch("/instrumentation/" + _name);
-    _radio_node = fgGetNode(branch.c_str(), _num, true);
+    string branch = nodePath();
+    _radio_node = fgGetNode(branch.c_str(), true);
 }
 
 
@@ -169,13 +168,11 @@ void
 FGNavRadio::init ()
 {
     SGPropertyNode* node = _radio_node.get();
-    bus_power_node = 
-        fgGetNode(("/systems/electrical/outputs/" + _name).c_str(), true);
-
+ 
+    initServicePowerProperties(node);
+    
     // inputs
     is_valid_node = node->getChild("data-is-valid", 0, true);
-    power_btn_node = node->getChild("power-btn", 0, true);
-    power_btn_node->setBoolValue( true );
     vol_btn_node = node->getChild("volume", 0, true);
     ident_btn_node = node->getChild("ident", 0, true);
     ident_btn_node->setBoolValue( true );
@@ -261,7 +258,7 @@ FGNavRadio::init ()
     _magvarNode = fgGetNode("/environment/magnetic-variation-deg", true);
     
     std::ostringstream temp;
-    temp << _name << "-ident-" << _num;
+    temp << name() << "-ident-" << number();
     if( NULL == _audioIdent ) 
         _audioIdent = new VORAudioIdent( temp.str() );
     _audioIdent->init();
@@ -278,20 +275,6 @@ FGNavRadio::reinit ()
     _time_before_search_sec = -1.0;
 }
 
-void
-FGNavRadio::bind ()
-{
-    _radio_node->tie( "operable", SGRawValueMethods<FGNavRadio,bool>( *this, &FGNavRadio::isOperable ) );
-}
-
-
-void
-FGNavRadio::unbind ()
-{
-    _radio_node->untie("operable");
-}
-
-
 // model standard VOR/DME/TACAN service volumes as per AIM 1-1-8
 double FGNavRadio::adjustNavRange( double stationElev, double aircraftElev,
                                  double nominalRange )
@@ -379,11 +362,8 @@ FGNavRadio::update(double dt)
     return; // paused
   }
     
-  if (power_btn_node->getBoolValue() 
-      && (bus_power_node->getDoubleValue() > 1.0)
-      && nav_serviceable_node->getBoolValue() )
+  if (isServiceableAndPowered())
   {
-    _operable = true;
     updateReceiver(dt);
     updateCDI(dt);
   } else {
@@ -428,7 +408,6 @@ void FGNavRadio::clearOutputs()
   is_valid_node->setBoolValue(false);
   nav_id_node->setStringValue("");
   
-  _operable = false;
   _navaid = NULL;
 }
 
@@ -868,8 +847,7 @@ void FGNavRadio::updateAudio( double dt )
   
 	// play station ident via audio system if on + ident,
 	// otherwise turn it off
-  if (!power_btn_node->getBoolValue()
-      || !(bus_power_node->getDoubleValue() > 1.0)
+  if (!isServiceableAndPowered()
       || !ident_btn_node->getBoolValue()
       || !audio_btn_node->getBoolValue() ) {
     _audioIdent->setIdent("", 0.0 );
@@ -989,3 +967,11 @@ void FGNavRadio::updateNav()
   id_c3_node->setIntValue( (int)identBuffer[2] );
   id_c4_node->setIntValue( (int)identBuffer[3] );
 }
+
+
+// Register the subsystem.
+#if 0
+SGSubsystemMgr::InstancedRegistrant<FGNavRadio> registrantFGNavRadio(
+    SGSubsystemMgr::FDM,
+    {{"instrumentation", SGSubsystemMgr::Dependency::HARD}});
+#endif
