@@ -30,6 +30,7 @@
 #include <cstring>             // strcmp()
 
 #if defined(SG_WINDOWS)
+#define _WINSOCKAPI_
 #  include <io.h>               // isatty()
 #  include <process.h>          // _getpid()
 #  include <Windows.h>
@@ -39,6 +40,7 @@
 #  include <sys/types.h>        
 #  include <sys/stat.h>
 #  include <fcntl.h>
+#  include <sys/file.h>
 #endif
 
 #include <string>
@@ -117,6 +119,9 @@
 
 #include <Traffic/TrafficMgr.hxx>
 #include <MultiPlayer/multiplaymgr.hxx>
+#if defined(ENABLE_SWIFT)
+#include <Network/Swift/swift_connection.hxx>
+#endif
 #include <FDM/fdm_shell.hxx>
 #include <Environment/ephemeris.hxx>
 #include <Environment/environment_mgr.hxx>
@@ -146,6 +151,7 @@
 #include "positioninit.hxx"
 #include "util.hxx"
 #include "AircraftDirVisitorBase.hxx"
+#include <Main/sentryIntegration.hxx>
 
 #if defined(SG_MAC)
 #include <GUI/CocoaHelpers.h> // for Mac impl of platformDefaultDataPath()
@@ -341,6 +347,7 @@ private:
   
   virtual VisitResult visit(const SGPath& p)
   {
+      SGPath realPath = p.realpath();
     // create cache node
     int i = 0;
     while (1) {
@@ -350,17 +357,17 @@ private:
     
     SGPropertyNode *n, *entry = _cache->getChild("aircraft", --i, true);
 
-    std::string fileName(p.file());
+    std::string fileName(realPath.file());
     n = entry->getNode("file", true);
     n->setStringValue(fileName);
     n->setAttribute(SGPropertyNode::USERARCHIVE, true);
 
     n = entry->getNode("path", true);
-    n->setStringValue(p.dir());
+    n->setStringValue(realPath.dir());
     n->setAttribute(SGPropertyNode::USERARCHIVE, true);
 
     if (simgear::strutils::iequals(fileName, _searchAircraft)) {
-        _foundPath = p;
+        _foundPath = realPath;
         return VISIT_DONE;
     }
 
@@ -384,7 +391,27 @@ private:
       } else {
           SG_LOG(SG_AIRCRAFT, SG_DEV_ALERT, "Aircraft does not specify a minimum FG version: please add one at /sim/minimum-fg-version");
       }
-
+      
+      auto compatNodes = globals->get_props()->getNode("/sim")->getChildren("compatible-fg-version");
+      if (!compatNodes.empty()) {
+          bool showCompatWarning = true;
+          
+          // if we have at least one compatibility node, then it needs to match
+          for (const auto& cn : compatNodes) {
+              const auto v = cn->getStringValue();
+              if (simgear::strutils::compareVersionToWildcard(FLIGHTGEAR_VERSION, v)) {
+                  showCompatWarning = false;
+                  break;
+              }
+          }
+          
+          if (showCompatWarning) {
+              flightgear::modalMessageBox("Aircraft not compatible with this version",
+              "The selected aircraft has not been checked for compatability with this version of FlightGear (" FLIGHTGEAR_VERSION  "). "
+                                          "Some aircraft features might not work, or might be displayed incorrectly.");
+          }
+      }
+      
       return true;
   }
   
@@ -424,9 +451,14 @@ static SGPath platformDefaultDataPath()
 static HANDLE static_fgHomeWriteMutex = nullptr;
 #endif
 
+SGPath fgHomePath()
+{
+    return SGPath::fromEnv("FG_HOME", platformDefaultDataPath());
+}
+
 bool fgInitHome()
 {
-  SGPath dataPath = SGPath::fromEnv("FG_HOME", platformDefaultDataPath());
+  SGPath dataPath = fgHomePath();
   globals->set_fg_home(dataPath);
     
     simgear::Dir fgHome(dataPath);
@@ -469,35 +501,61 @@ bool fgInitHome()
 	}
 #else
 // write our PID, and check writeability
-    SGPath pidPath(dataPath, "fgfs.pid");
+    SGPath pidPath(dataPath, "fgfs_lock.pid");
+    std::string ps = pidPath.utf8Str();
+
     if (pidPath.exists()) {
-        SG_LOG(SG_GENERAL, SG_ALERT, "flightgear instance already running, switching to FG_HOME read-only.");
-        // set a marker property so terrasync/navcache don't try to write
-        // from secondary instances
-        fgSetBool("/sim/fghome-readonly", true);
-        return true;
-    }
+        int fd = ::open(ps.c_str(), O_RDONLY, 0644);
+        if (fd < 0) {
+            SG_LOG(SG_GENERAL, SG_ALERT, "failed to open local file:" << pidPath
+                   << "\n\tdue to:" << simgear::strutils::error_string(errno));
+            return false;
+        }
+        
+        int err = ::flock(fd, LOCK_EX | LOCK_NB);
+        if (err < 0) {
+            if ( errno ==  EWOULDBLOCK) {
+                SG_LOG(SG_GENERAL, SG_ALERT, "flightgear instance already running, switching to FG_HOME read-only.");
+                // set a marker property so terrasync/navcache don't try to write
+                // from secondary instances
+                fgSetBool("/sim/fghome-readonly", true);
+                return true;
+            } else {
+                SG_LOG(SG_GENERAL, SG_ALERT, "failed to lock file:" << pidPath
+                       << "\n\tdue to:" << simgear::strutils::error_string(errno));
+                return false;
+            }
+        }
+        
+       // we locked it!
+        result = true;
+    } else {
+        char buf[16];
+       std::string ps = pidPath.utf8Str();
 
-    char buf[16];
-    std::string ps = pidPath.local8BitStr();
+       ssize_t len = snprintf(buf, 16, "%d\n", getpid());
+       int fd = ::open(ps.c_str(), O_WRONLY | O_CREAT | O_TRUNC, 0644);
+        if (fd < 0) {
+            SG_LOG(SG_GENERAL, SG_ALERT, "failed to open local file:" << pidPath
+               << "\n\tdue to:" << simgear::strutils::error_string(errno));
+            return false;
+        }
+            
+        int err = write(fd, buf, len);
+        if (err < 0) {
+            SG_LOG(SG_GENERAL, SG_ALERT, "failed to write to lock file:" << pidPath
+            << "\n\tdue to:" << simgear::strutils::error_string(errno));
+            return false;
+        }
 
-    // do open+unlink trick to the file is deleted on exit, even if we
-    // crash or exit(-1)
-    ssize_t len = snprintf(buf, 16, "%d", getpid());
-    int fd = ::open(ps.c_str(), O_WRONLY | O_CREAT | O_TRUNC | O_EXCL, 0644);
-    if (fd >= 0) {
-        result = ::write(fd, buf, len) == len;
-        if( ::unlink(ps.c_str()) != 0 ) // delete file when app quits
-          result = false;
-    }
-
-    if (!result) {
-        flightgear::fatalMessageBoxWithoutExit(
-            "File permissions problem",
-            "Can't write to user-data storage folder, check file permissions "
-            "and FG_HOME.",
-            "User-data at '" + dataPath.utf8Str() + "'.");
-        return false;
+        err = flock(fd, LOCK_EX);
+        if (err != 0) {
+            SG_LOG(SG_GENERAL, SG_ALERT, "failed to lock file:" << pidPath
+            << "\n\tdue to:" << simgear::strutils::error_string(errno));
+            return false;
+        }
+        
+        result = true;
     }
 #endif
     fgSetBool("/sim/fghome-readonly", false);
@@ -510,6 +568,11 @@ void fgShutdownHome()
 	if (static_fgHomeWriteMutex) {
 		CloseHandle(static_fgHomeWriteMutex);
 	}
+#else
+    if (fgGetBool("/sim/fghome-readonly") == false) {
+        SGPath pidPath = globals->get_fg_home() / "fgfs_lock.pid";
+        pidPath.remove();
+    }
 #endif
 }
 
@@ -528,6 +591,17 @@ static void createBaseStorageDirForAddons(const SGPath& exportDir)
     }
 }
 
+struct SimLogFileLine : SGPropertyChangeListener
+{
+    SimLogFileLine() {
+        fgAddChangeListener(this, "/sim/log-file-line");
+    }
+    virtual void valueChanged(SGPropertyNode* node) {
+        bool    fileLine = node->getBoolValue();
+        sglog().setFileLine(fileLine);
+    }
+};
+
 // Read in configuration (file and command line)
 int fgInitConfig ( int argc, char **argv, bool reinit )
 {
@@ -554,7 +628,10 @@ int fgInitConfig ( int argc, char **argv, bool reinit )
     fgSetDefaults();
     flightgear::Options* options = flightgear::Options::sharedInstance();
     if (!reinit) {
-        options->init(argc, argv, dataPath);
+        auto result = options->init(argc, argv, dataPath);
+        if (result != flightgear::FG_OPTIONS_OK) {
+            return result;
+        }
     }
 
     // establish default for developer-mode based upon compiled build types
@@ -569,11 +646,22 @@ int fgInitConfig ( int argc, char **argv, bool reinit )
         developerMode = simgear::strutils::to_bool(s);
     }
 
-    fgSetBool("/sim/developer-mode", developerMode);
+    auto node = fgGetNode("/sim/developer-mode", true);
+    // ensure this value survives reset
+    node->setAttribute(SGPropertyNode::PRESERVE, true);
+    node->setBoolValue(developerMode);
     sglog().setDeveloperMode(developerMode);
+    
+    static SimLogFileLine   simLogFileLine;
+
+#ifdef ENABLE_SWIFT
+    //Set standard settings for swift connection
+    fgSetString("/sim/swift/adress","127.0.0.1");
+    fgSetString("/sim/swift/port","45003");
+#endif
 
     // Read global defaults from $FG_ROOT/defaults
-    SG_LOG(SG_GENERAL, SG_INFO, "Reading global defaults");
+    SG_LOG(SG_GENERAL, SG_DEBUG, "Reading global defaults");
     SGPath defaultsXML = globals->get_fg_root() / "defaults.xml";
     if (!defaultsXML.exists()) {
         flightgear::fatalMessageBoxThenExit(
@@ -633,7 +721,9 @@ void fgInitAircraftPaths(bool reinit)
 int fgInitAircraft(bool reinit)
 {
     if (!reinit) {
-        flightgear::Options::sharedInstance()->initAircraft();
+        auto r = flightgear::Options::sharedInstance()->initAircraft();
+        if (r == flightgear::FG_OPTIONS_SHOW_AIRCRAFT)
+            return r;
     }
     
     FindAndCacheAircraft f(globals->get_props());
@@ -643,6 +733,8 @@ int fgInitAircraft(bool reinit)
     SGPropertyNode* aircraftProp = fgGetNode("/sim/aircraft", true);
     string aircraftId(aircraftProp->getStringValue());
 
+    flightgear::addSentryTag("aircraft", aircraftId);
+        
     PackageRef acftPackage;
     if (!haveExplicit) {
         acftPackage = pkgRoot->getPackageById(aircraftId);
@@ -850,8 +942,7 @@ void fgCreateSubsystems(bool duringReset) {
 
     SGPath mpath( globals->get_fg_root() );
     mpath.append( fgGetString("/sim/rendering/materials-file") );
-    if ( ! globals->get_matlib()->load(globals->get_fg_root().local8BitStr(), mpath.local8BitStr(),
-            globals->get_props()) ) {
+    if ( ! globals->get_matlib()->load(globals->get_fg_root(), mpath, globals->get_props()) ) {
        throw sg_io_exception("Error loading materials file", mpath);
     }
 
@@ -937,6 +1028,14 @@ void fgCreateSubsystems(bool duringReset) {
 
     globals->add_subsystem("mp", new FGMultiplayMgr, SGSubsystemMgr::POST_FDM);
 
+#ifdef ENABLE_SWIFT
+    ////////////////////////////////////////////////////////////////////
+    // Initialize Swift subsystem
+    ////////////////////////////////////////////////////////////////////
+
+    globals->add_subsystem("swift", new SwiftConnection, SGSubsystemMgr::POST_FDM);
+#endif
+
     ////////////////////////////////////////////////////////////////////
     // Initialise the AI Model Manager
     ////////////////////////////////////////////////////////////////////
@@ -1049,6 +1148,8 @@ void fgStartReposition()
   SGPropertyNode *master_freeze = fgGetNode("/sim/freeze/master");
   SG_LOG( SG_GENERAL, SG_INFO, "fgStartReposition()");
   
+  flightgear::addSentryBreadcrumb("start reposition", "info");
+
   // ensure we are frozen
   bool freeze = master_freeze->getBoolValue();
   if ( !freeze ) {
@@ -1067,8 +1168,7 @@ void fgStartReposition()
   // main-loop
   flightgear::initPosition();
   
-  simgear::SGTerraSync* terraSync =
-  static_cast<simgear::SGTerraSync*>(globals->get_subsystem("terrasync"));
+  auto terraSync = globals->get_subsystem<simgear::SGTerraSync>();
   if (terraSync) {
     terraSync->reposition();
   }
@@ -1135,7 +1235,7 @@ void fgStartNewReset()
         SGSubsystemGroup* grp = subsystemManger->get_group(static_cast<SGSubsystemMgr::GroupType>(g));
         for (auto nm : grp->member_names()) {
             if ((nm == "time") || (nm == "terrasync") || (nm == "events")
-                || (nm == "lighting") || (nm == FGScenery::subsystemName()))
+                || (nm == "lighting") || (nm == FGScenery::staticSubsystemClassId()))
             {
                 continue;
             }
@@ -1158,7 +1258,7 @@ void fgStartNewReset()
     
     // order is important here since tile-manager shutdown needs to
     // access the scenery object
-    subsystemManger->remove(FGScenery::subsystemName());
+    subsystemManger->remove(FGScenery::staticSubsystemClassId());
 
     FGScenery::getPagerSingleton()->clearRequests();
     flightgear::CameraGroup::setDefault(NULL);
@@ -1194,6 +1294,9 @@ void fgStartNewReset()
     fgInitConfig(0, NULL, true);
     fgInitGeneral(); // all of this?
     
+    // set out new property root on the command manager
+    SGCommandMgr::instance()->setImplicitRoot(globals->get_props());
+    
     flightgear::Options::sharedInstance()->processOptions();
 
     // Rebuild the lists of allowed paths for cases where a path comes from an
