@@ -230,16 +230,6 @@ setFreeze (bool f)
 {
     frozen = f;
 
-    // Stop sound on a pause
-    SGSoundMgr *smgr = globals->get_subsystem<SGSoundMgr>();
-    if ( smgr != NULL ) {
-        if ( f ) {
-            smgr->suspend();
-        } else if (fgGetBool("/sim/sound/working")) {
-            smgr->resume();
-        }
-    }
-
     // Pause the particle system
     simgear::Particles::setFrozen(f);
 }
@@ -336,28 +326,6 @@ getGMTString ()
   return buf;
 }
 
-/**
- * Return the current heading in degrees.
- */
-static double
-getHeadingMag ()
-{
-  double magheading = fgGetDouble("/orientation/heading-deg") -
-    fgGetDouble("/environment/magnetic-variation-deg");
-  return SGMiscd::normalizePeriodic(0, 360, magheading );
-}
-
-/**
- * Return the current track in degrees.
- */
-static double
-getTrackMag ()
-{
-  double magtrack = fgGetDouble("/orientation/track-deg") -
-    fgGetDouble("/environment/magnetic-variation-deg");
-  return SGMiscd::normalizePeriodic(0, 360, magtrack );
-}
-
 ////////////////////////////////////////////////////////////////////////
 // Tie the properties.
 ////////////////////////////////////////////////////////////////////////
@@ -405,6 +373,15 @@ FGProperties::init ()
 {
 }
 
+static SGPropertyNode_ptr initDoubleNode(const std::string& path, const double v)
+{
+    auto r = fgGetNode(path, true);
+    if (r->getType() == simgear::props::NONE) {
+        r->setDoubleValue(v);
+    }
+    return r;
+}
+
 void
 FGProperties::bind ()
 {
@@ -449,9 +426,11 @@ FGProperties::bind ()
   _tiedProperties.Tie<const char*>("/position/latitude-string", getLatitudeString);
   _tiedProperties.Tie<const char*>("/position/longitude-string", getLongitudeString);
 
-  // Orientation
-  _tiedProperties.Tie<double>("/orientation/heading-magnetic-deg", getHeadingMag);
-  _tiedProperties.Tie<double>("/orientation/track-magnetic-deg", getTrackMag);
+    _headingMagnetic = initDoubleNode("/orientation/heading-magnetic-deg", 0.0);
+    _trackMagnetic = initDoubleNode("/orientation/track-magnetic-deg", 0.0);
+    _magVar = initDoubleNode("/environment/magnetic-variation-deg", 0.0);
+    _trueHeading = initDoubleNode("/orientation/heading-deg", 0.0);
+    _trueTrack = initDoubleNode("/orientation/track-deg", 0.0);
 }
 
 void
@@ -491,9 +470,19 @@ FGProperties::update (double dt)
     _rmin->setIntValue(r->tm_min);
     _rsec->setIntValue(r->tm_sec);
     _rwday->setIntValue(r->tm_wday);
+    
+    const double magvar = _magVar->getDoubleValue();
+    const auto hdgMag = SGMiscd::normalizePeriodic(0, 360.0, _trueHeading->getDoubleValue() - magvar);
+    _headingMagnetic->setDoubleValue(hdgMag);
+    
+    const auto trackMag = SGMiscd::normalizePeriodic(0, 360.0, _trueTrack->getDoubleValue() - magvar);
+    _trackMagnetic->setDoubleValue(trackMag);
 }
 
 
+// Register the subsystem.
+SGSubsystemMgr::Registrant<FGProperties> registrantFGProperties;
+
 
 ////////////////////////////////////////////////////////////////////////
 // Save and restore.
