@@ -133,6 +133,17 @@ public:
   }
 };
 
+class CompositorEffectsProvider : public simgear::ResourceProvider {
+public:
+    CompositorEffectsProvider() :
+        simgear::ResourceProvider(simgear::ResourceManager::PRIORITY_NORMAL) {
+    }
+    SGPath resolve(const std::string &aResource, SGPath&) const override {
+        const SGPath p = globals->get_fg_root() / "Compositor" / aResource;
+        return p.exists() ? p : SGPath();
+    }
+};
+
 ////////////////////////////////////////////////////////////////////////
 // Implementation of FGGlobals.
 ////////////////////////////////////////////////////////////////////////
@@ -144,7 +155,7 @@ FGGlobals *globals = NULL;
 // Constructor
 FGGlobals::FGGlobals() :
     renderer( new FGRenderer ),
-    subsystem_mgr( new SGSubsystemMgr("subsystem_mgr") ),
+    subsystem_mgr( new SGSubsystemMgr ),
     event_mgr( new SGEventMgr ),
     sim_time_sec( 0.0 ),
     fg_root( "" ),
@@ -165,6 +176,9 @@ FGGlobals::FGGlobals() :
     resMgr->addProvider(new AircraftResourceProvider());
     resMgr->addProvider(new CurrentAircraftDirProvider());
     resMgr->addProvider(new flightgear::addons::ResourceProvider());
+#ifdef ENABLE_COMPOSITOR
+    resMgr->addProvider(new CompositorEffectsProvider());
+#endif
     initProperties();
 }
 
@@ -213,7 +227,7 @@ FGGlobals::~FGGlobals()
     }
 
     osgDB::Registry::instance()->clearObjectCache();
-    subsystem_mgr->remove(FGScenery::subsystemName());
+    subsystem_mgr->remove(FGScenery::staticSubsystemClassId());
 
     // renderer touches subsystems during its destruction
     set_renderer(NULL);
@@ -364,9 +378,7 @@ void FGGlobals::append_fg_scenery (const SGPath &path)
 
     // tell the ResouceManager about the scenery path
     // needed to load Models from this scenery path
-    simgear::ResourceManager::instance()->addBasePath(abspath.local8BitStr(),
-                                                      simgear::ResourceManager::PRIORITY_DEFAULT);
-    simgear::Dir dir(abspath);
+    simgear::ResourceManager::instance()->addBasePath(abspath, simgear::ResourceManager::PRIORITY_DEFAULT);
 
     fg_scenery.push_back(abspath);
     extra_read_allowed_paths.push_back(abspath);
@@ -384,7 +396,7 @@ void FGGlobals::append_read_allowed_paths(const SGPath &path)
 {
     SGPath abspath(path.realpath());
     if (!abspath.exists()) {
-        SG_LOG(SG_GENERAL, SG_WARN, "read-allowed path not found:" << abspath);
+        SG_LOG(SG_IO, SG_DEBUG, "read-allowed path not found:" << abspath);
         return;
     }
     extra_read_allowed_paths.push_back(abspath);
@@ -764,15 +776,26 @@ static void tryAutosaveMigration(const SGPath& userDataPath, SGPropertyNode* pro
     // apply migration filters for each property in turn
     deleteProperties(&oldProps, blacklist);
 
+    // manual migrations
+    // migrate pre-2019.1 sense of /sim/mouse/invert-mouse-wheel
+    if (foundVersion.first < 2019) {
+        SGPropertyNode_ptr wheelNode = oldProps.getNode("/sim/mouse/invert-mouse-wheel");
+        if (wheelNode) {
+            wheelNode->setBoolValue(!wheelNode->getBoolValue());
+        }
+    }
     // copy remaining props out
     copyProperties(&oldProps, props);
 
     // inform the user
-    flightgear::modalMessageBox("Settings migrated",
-                                "Saved settings were migrated from a previous version of FlightGear. "
-                                "If you encounter any problems when using the system, try restoring "
-                                "the default settings, before reporting a problem. "
-                                "Saved settings can affect the appearance, performance and features of the simulator.");
+    FGLocale *locale = globals->get_locale();
+    const auto title = locale->getLocalizedString("settings-migration-title", "sys", "Settings migrated");
+    const auto msg = locale->getLocalizedString("settings-migration-text", "sys",
+                                                "Saved settings were migrated from a previous version of FlightGear. "
+                                                "If you encounter any problems when using the system, try restoring "
+                                                "the default settings, before reporting a problem. "
+                                                "Saved settings can affect the appearance, performance and features of the simulator.");
+    flightgear::modalMessageBox(title, msg);
 }
 
 // Load user settings from the autosave file (normally in $FG_HOME)
@@ -800,6 +823,13 @@ FGGlobals::loadUserSettings(SGPath userDataPath)
     } else {
         tryAutosaveMigration(userDataPath, &autosave);
     }
+    /* Before 2020-03-10, we could save portions of the /ai/models/ tree, which
+    confuses things when loaded again. So delete any such items if they have
+    been loaded. */
+    SGPropertyNode* ai = autosave.getNode("ai");
+    if (ai) {
+        ai->removeChildren("models");
+    }
     copyProperties(&autosave, globals->get_props());
 }
 
