@@ -20,10 +20,7 @@
 //
 // $Id$
 
-
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include <config.h>
 
 #include <simgear/compiler.h>
 #include <simgear/props/props_io.hxx>
@@ -34,13 +31,6 @@
 #include <osg/GraphicsContext>
 #include <osgDB/Registry>
 
-#if defined(HAVE_CRASHRPT)
-	#include <CrashRpt.h>
-
-// defined in bootstrap.cxx
-extern bool global_crashRptEnabled;
-
-#endif
 
 // Class references
 #include <simgear/canvas/VGInitOperation.hxx>
@@ -55,6 +45,9 @@ extern bool global_crashRptEnabled;
 #include <simgear/math/SGMath.hxx>
 #include <simgear/math/sg_random.h>
 #include <simgear/misc/strutils.hxx>
+#include <simgear/structure/commands.hxx>
+#include <simgear/emesary/Emesary.hxx>
+#include <simgear/emesary/notifications.hxx>
 
 #include <Add-ons/AddonManager.hxx>
 #include <Main/locale.hxx>
@@ -82,6 +75,7 @@ extern bool global_crashRptEnabled;
 #include "screensaver_control.hxx"
 #include "subsystemFactory.hxx"
 #include "options.hxx"
+#include <Main/sentryIntegration.hxx>
 
 #include <simgear/embedded_resources/EmbeddedResourceManager.hxx>
 #include <EmbeddedResources/FlightGear-resources.hxx>
@@ -90,6 +84,10 @@ extern bool global_crashRptEnabled;
 #include <GUI/QtLauncher.hxx>
 #endif
 
+#ifdef __OpenBSD__
+#include <sys/resource.h>
+#endif
+
 using namespace flightgear;
 
 using std::cerr;
@@ -100,11 +98,52 @@ using std::vector;
 extern int _bootstrap_OSInit;
 
 static SGPropertyNode_ptr frame_signal;
+static SGPropertyNode_ptr nasal_gc_threaded;
+static SGPropertyNode_ptr nasal_gc_threaded_wait;
 
-// What should we do when we have nothing else to do?  Let's get ready
-// for the next move and update the display?
+#ifdef NASAL_BACKGROUND_GC_THREAD
+extern "C" {
+    extern void startNasalBackgroundGarbageCollection();
+    extern void stopNasalBackgroundGarbageCollection();
+    extern void performNasalBackgroundGarbageCollection();
+    extern void awaitNasalGarbageCollectionComplete(bool can_wait);
+}
+#endif
+static  simgear::Notifications::MainLoopNotification mln_begin(simgear::Notifications::MainLoopNotification::Type::Begin);
+static  simgear::Notifications::MainLoopNotification mln_end(simgear::Notifications::MainLoopNotification::Type::End);
+static  simgear::Notifications::MainLoopNotification mln_started(simgear::Notifications::MainLoopNotification::Type::Started);
+static  simgear::Notifications::MainLoopNotification mln_stopped(simgear::Notifications::MainLoopNotification::Type::Stopped);
+static  simgear::Notifications::NasalGarbageCollectionConfigurationNotification *ngccn = nullptr;
+// This method is usually called after OSG has finished rendering a frame in what OSG calls an idle handler and
+// is reposonsible for invoking all of the relevant per frame processing; most of which is handled by subsystems.
 static void fgMainLoop( void )
 {
+    //
+    // the Nasal GC will automatically run when (during allocation) it discovers that more space is needed.
+    // This has a cost of between 5ms and 50ms (depending on the amount of currently active Nasal).
+    // The result is unscheduled and unpredictable pauses during normal operation when the garbage collector
+    // runs; which typically occurs at intervals between 1sec and 20sec.
+    // 
+    // The solution to this, which overall increases CPU load, is to use a thread to do this; as Nasal is thread safe
+    // so what we do is to launch the garbage collection at the end of the main loop and then wait for completion at the start of the
+    // next main loop.
+    // So although the overall CPU is increased it has little effect on the frame rate; if anything it is an overall benefit
+    // as there are no unscheduled long duration frames.
+    //
+    // The implementation appears to work fine without waiting for completion at the start of the frame - so
+    // this wait at the start can be disabled by setting the property /sim/nasal-gc-threaded-wait to false.
+
+    // first we see if the config has changed. The notification will return true from SetActive/SetWait when the
+    // value has been changed - and thus we notify the Nasal system that it should configure itself accordingly.
+    auto use_threaded_gc = nasal_gc_threaded->getBoolValue();
+    auto threaded_wait = nasal_gc_threaded_wait->getBoolValue();
+    bool notify_gc_config = false;
+    notify_gc_config = ngccn->SetActive(use_threaded_gc);
+    notify_gc_config |= ngccn->SetWait(threaded_wait);
+    if (notify_gc_config)
+         simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(*ngccn);
+
+     simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(mln_begin);
 
     if (sglog().has_popup()) {
         std::string s = sglog().get_popup();
@@ -122,7 +161,11 @@ static void fgMainLoop( void )
     // update all subsystems
     globals->get_subsystem_mgr()->update(sim_dt);
 
+    // flush commands waiting in the queue
+    SGCommandMgr::instance()->executedQueuedCommands();
     simgear::AtomicChangeListener::fireChangeListeners();
+
+     simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(mln_end);
 }
 
 static void initTerrasync()
@@ -162,10 +205,18 @@ static void initTerrasync()
 
     terra_sync->bind();
     terra_sync->init();
+    
+    if (fgGetBool("/sim/terrasync/enabled")) {
+        flightgear::addSentryTag("terrasync", "enabled");
+    }
 }
 
 static void fgSetVideoOptions()
 {
+    SGPath userDataPath = globals->get_fg_home();
+    SGPath autosaveFile = globals->autosaveFilePath(userDataPath);
+    if (autosaveFile.exists()) return;
+
     std::string vendor = fgGetString("/sim/rendering/gl-vendor");
     SGPath path(globals->get_fg_root());
     path.append("Video");
@@ -193,17 +244,16 @@ static void fgSetVideoOptions()
                 SG_LOG(SG_INPUT, SG_WARN, "failed to read video settings:" << e.getMessage()
                 << "(from " << e.getOrigin() << ")");
             }
-
-            flightgear::Options* options = flightgear::Options::sharedInstance();
-            if (!options->isOptionSet("ignore-autosave")) {
-                globals->loadUserSettings();
-            }
         }
     }
 }
 
 static void checkOpenGLVersion()
 {
+    flightgear::addSentryTag("gl-version", fgGetString("/sim/rendering/gl-version"));
+    flightgear::addSentryTag("gl-renderer", fgGetString("/sim/rendering/gl-vendor"));
+    flightgear::addSentryTag("gl-vendor", fgGetString("/sim/rendering/gl-renderer"));
+    
 #if defined(SG_MAC)
     // Mac users can't upgrade their drivers, so complaining about
     // versions doesn't help them much
@@ -237,6 +287,8 @@ static void registerMainLoop()
 {
     // stash current frame signal property
     frame_signal = fgGetNode("/sim/signals/frame", true);
+    nasal_gc_threaded = fgGetNode("/sim/nasal-gc-threaded", true);
+    nasal_gc_threaded_wait = fgGetNode("/sim/nasal-gc-threaded-wait", true);
     fgRegisterIdleHandler( fgMainLoop );
 }
 
@@ -290,7 +342,7 @@ static void fgIdleFunction ( void ) {
         // Initialize the property-based built-in commands
         ////////////////////////////////////////////////////////////////////
         fgInitCommands();
-				fgInitSceneCommands();
+        fgInitSceneCommands();
 
         flightgear::registerSubsystemCommands(globals->get_commands());
 
@@ -305,7 +357,7 @@ static void fgIdleFunction ( void ) {
         flightgear::initPosition();
         flightgear::initTowerLocationListener();
 
-        simgear::SGModelLib::init(globals->get_fg_root().local8BitStr(), globals->get_props());
+        simgear::SGModelLib::init(globals->get_fg_root().utf8Str(), globals->get_props());
 
         auto timeManager = globals->get_subsystem<TimeManager>();
         timeManager->init();
@@ -373,9 +425,16 @@ static void fgIdleFunction ( void ) {
         // run the main loop.
         fgSetBool("sim/sceneryloaded", false);
         registerMainLoop();
+        
+        ngccn = new simgear::Notifications::NasalGarbageCollectionConfigurationNotification(nasal_gc_threaded->getBoolValue(), nasal_gc_threaded_wait->getBoolValue());
+         simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(*ngccn);
+         simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(mln_started);
+        
+        flightgear::addSentryBreadcrumb("entering main loop", "info");
     }
 
     if ( idle_state == 2000 ) {
+        flightgear::addSentryBreadcrumb("starting reset", "info");
         fgStartNewReset();
         idle_state = 2005;
     }
@@ -460,15 +519,6 @@ static void logToHome()
         logPath.append("fgfs.log");
     }
     sglog().logToFile(logPath, SG_ALL, SG_INFO);
-
-#if defined(HAVE_CRASHRPT)
-	if (global_crashRptEnabled) {
-		crAddFile2(logPath.c_str(), NULL, "FlightGear Log File", CR_AF_MAKE_FILE_COPY);
-		SG_LOG( SG_GENERAL, SG_INFO, "CrashRpt enabled");
-	} else {
-		SG_LOG(SG_GENERAL, SG_WARN, "CrashRpt enabled at compile time but failed to install");
-	}
-#endif
 }
 
 // Main top level initialization
@@ -478,11 +528,15 @@ int fgMainInit( int argc, char **argv )
     // level once startup is done.
     sglog().setLogLevels( SG_ALL, SG_INFO );
     sglog().setStartupLoggingEnabled(true);
-
+    
     globals = new FGGlobals;
     if (!fgInitHome()) {
         return EXIT_FAILURE;
     }
+    
+#if defined(HAVE_QT)
+    flightgear::initApp(argc, argv);
+#endif
 
     const bool readOnlyFGHome = fgGetBool("/sim/fghome-readonly");
     if (!readOnlyFGHome) {
@@ -490,6 +544,10 @@ int fgMainInit( int argc, char **argv )
         logToHome();
     }
 
+    if (readOnlyFGHome) {
+        flightgear::addSentryTag("fghome-readonly", "true");
+    }
+
     std::string version(FLIGHTGEAR_VERSION);
     SG_LOG( SG_GENERAL, SG_INFO, "FlightGear:  Version " << version );
     SG_LOG( SG_GENERAL, SG_INFO, "FlightGear:  Build Type " << FG_BUILD_TYPE );
@@ -501,6 +559,32 @@ int fgMainInit( int argc, char **argv )
     SG_LOG(SG_GENERAL, SG_ALERT, "Minimum supported OpenScenegraph is V3.4.1 - currently using " << osgGetVersion() << " This can cause fatal OSG 'final reference count' errors at runtime");
 #endif
 
+    flightgear::addSentryTag("jenkins-build-number", std::to_string(JENKINS_BUILD_NUMBER));
+    flightgear::addSentryTag("osg-version", osgGetVersion());
+    
+#ifdef __OpenBSD__
+    {
+        /* OpenBSD defaults to a small maximum data segment, which can cause
+        flightgear to crash with SIGBUS, so output a warning if this is likely.
+        */
+        struct rlimit   rlimit;
+        int e = getrlimit(RLIMIT_DATA, &rlimit);
+        if (e) {
+            SG_LOG( SG_GENERAL, SG_INFO, "This is OpenBSD; getrlimit() failed: " << strerror(errno));
+        }
+        else {
+            long long   required = 4LL * (1LL<<30);
+            if (rlimit.rlim_cur < required) {
+                SG_LOG( SG_GENERAL, SG_POPUP, ""
+                        << "Max data segment (" << rlimit.rlim_cur << "bytes) too small.\n"
+                        << "This can cause Flightgear to crash due to SIGBUS.\n"
+                        << "E.g. increase with 'ulimit -d " << required/1024 << "'."
+                        );
+            }
+        }
+    }
+#endif
+
     // seed the random number generator
     sg_srandom_time();
 
@@ -544,12 +628,12 @@ int fgMainInit( int argc, char **argv )
 
 #if defined(HAVE_QT)
     if (showLauncher) {
-        flightgear::initApp(argc, argv);
-        flightgear::checkKeyboardModifiersForSettingFGRoot();
-
+        flightgear::addSentryBreadcrumb("starting launcher", "info");
         if (!flightgear::runLauncherDialog()) {
             return EXIT_SUCCESS;
         }
+
+        flightgear::addSentryBreadcrumb("completed launcher", "info");
     }
 #else
     if (showLauncher) {
@@ -563,7 +647,9 @@ int fgMainInit( int argc, char **argv )
     configResult = fgInitAircraft(false);
     if (configResult == flightgear::FG_OPTIONS_ERROR) {
         return EXIT_FAILURE;
-    } else if (configResult == flightgear::FG_OPTIONS_EXIT) {
+    } else if ((configResult == flightgear::FG_OPTIONS_EXIT) ||
+               (configResult == flightgear::FG_OPTIONS_SHOW_AIRCRAFT))
+    {
         return EXIT_SUCCESS;
     }
 
@@ -611,6 +697,7 @@ int fgMainInit( int argc, char **argv )
 
     if (fgGetBool("/sim/ati-viewport-hack", true)) {
         SG_LOG(SG_GENERAL, SG_WARN, "Enabling ATI/AMD viewport hack");
+        flightgear::addSentryTag("ati-viewport-hack", "enabled");
         ATIScreenSizeHack();
     }
 
@@ -623,6 +710,9 @@ int fgMainInit( int argc, char **argv )
     int result = fgOSMainLoop();
     frame_signal.clear();
     fgOSCloseWindow();
+    fgShutdownHome();
+    
+     simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(mln_stopped);
 
     simgear::clearEffectCache();
 
