@@ -26,9 +26,11 @@
 // std
 #include <cstddef>  // for std::size_t
 #include <map>
+#include <cstring>  // for memcoy
 #include <cassert>
 #include <stdint.h> // for int64_t
 #include <sstream>  // for std::ostringstream
+#include <mutex>
 
 #ifdef SYSTEM_SQLITE
 // the standard sqlite3.h doesn't give a way to set SQLITE_UINT64_TYPE,
@@ -53,7 +55,6 @@
 #include <simgear/misc/sg_dir.hxx>
 #include <simgear/misc/strutils.hxx>
 #include <simgear/threads/SGThread.hxx>
-#include <simgear/threads/SGGuard.hxx>
 
 #include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
@@ -171,7 +172,7 @@ public:
 
   bool isFinished() const
   {
-    SGGuard<SGMutex> g(_lock);
+    std::lock_guard<std::mutex> g(_lock);
     return _isFinished;
   }
 
@@ -182,7 +183,7 @@ public:
     _cache->doRebuild();
     SG_LOG(SG_NAVCACHE, SG_INFO, "cache rebuild took:" << st.elapsedMSec() << "msec");
 
-    SGGuard<SGMutex> g(_lock);
+    std::lock_guard<std::mutex> g(_lock);
     _isFinished = true;
     _phase = NavDataCache::REBUILD_DONE;
   }
@@ -190,7 +191,7 @@ public:
     NavDataCache::RebuildPhase currentPhase() const
     {
         NavDataCache::RebuildPhase ph;
-        SGGuard<SGMutex> g(_lock);
+        std::lock_guard<std::mutex> g(_lock);
         ph = _phase;
         return ph;
     }
@@ -198,14 +199,14 @@ public:
     unsigned int completionPercent() const
     {
         unsigned int perc = 0;
-        SGGuard<SGMutex> g(_lock);
+        std::lock_guard<std::mutex> g(_lock);
         perc = _completionPercent;
         return perc;
     }
 
     void setProgress(NavDataCache::RebuildPhase ph, unsigned int percent)
     {
-        SGGuard<SGMutex> g(_lock);
+        std::lock_guard<std::mutex> g(_lock);
         _phase = ph;
         _completionPercent = percent;
     }
@@ -214,7 +215,7 @@ private:
   NavDataCache* _cache;
     NavDataCache::RebuildPhase _phase;
     unsigned int _completionPercent;
-  mutable SGMutex _lock;
+  mutable std::mutex _lock;
   bool _isFinished;
 };
 
@@ -1250,12 +1251,12 @@ NavDataCache::NavDataCache()
             }
         }
     } // of retry loop
-    
+
     double RADIUS_EARTH_M = 7000 * 1000.0; // 7000km is plenty
     SGVec3d earthExtent(RADIUS_EARTH_M, RADIUS_EARTH_M, RADIUS_EARTH_M);
     Octree::global_spatialOctree =
     new Octree::Branch(SGBox<double>(-earthExtent, earthExtent), 1);
-    
+
     // Update d->aptDatFilesInfo, d->metarDatPath, d->navDatPath, etc.
     updateListsOfDatFiles();
 }
@@ -1263,12 +1264,17 @@ NavDataCache::NavDataCache()
 NavDataCache::~NavDataCache()
 {
   assert(static_instance == this);
-  static_instance = NULL;
+  static_instance = nullptr;
   d.reset();
+
+// ensure we wip the airports cache too, or we'll get out
+// of sync during tests
+  FGAirport::clearAirportsCache();
 }
 
 NavDataCache* NavDataCache::createInstance()
 {
+    assert(static_instance == nullptr);
     static_instance = new NavDataCache;
     return static_instance;
 }
@@ -2037,7 +2043,7 @@ char** NavDataCache::searchAirportNamesAndIdents(const std::string& searchInput)
   string heliport("HELIPORT");
   bool heli_p = searchInput.substr(0, heliport.length()) == heliport;
   auto pos = searchInput.find(":");
-  string aFilter((pos != string::npos) ? searchInput.substr(pos+1) : searchInput);  
+  string aFilter((pos != string::npos) ? searchInput.substr(pos+1) : searchInput);
   string searchTerm("%" + aFilter + "%");
 
   if (aFilter.empty() && !heli_p) {
@@ -2314,24 +2320,24 @@ AirwayEdgeVec NavDataCache::airwayEdgesFrom(int network, PositionedID pos)
   }
 
   d->reset(d->airwayEdgesFrom);
-    
+
 // find bidirectional / backwsards edges
     // at present all edges are bidirectional
     sqlite3_bind_int(d->airwayEdgesTo, 1, network);
     sqlite3_bind_int64(d->airwayEdgesTo, 2, pos);
-    
+
     while (d->stepSelect(d->airwayEdgesTo)) {
         result.push_back(AirwayEdge(
                                     sqlite3_column_int(d->airwayEdgesTo, 0),
                                     sqlite3_column_int64(d->airwayEdgesTo, 1)
                                     ));
     }
-    
+
     d->reset(d->airwayEdgesTo);
-    
+
   return result;
 }
-    
+
 AirwayRef NavDataCache::loadAirway(int airwayID)
 {
     sqlite3_bind_int(d->loadAirway, 1, airwayID);
@@ -2370,7 +2376,7 @@ PositionedIDVec NavDataCache::airwayWaypts(int id)
 
 // linearize
     PositionedIDVec result;
-    
+
     while (!rawEdges.empty()) {
         bool didAddEdge = false;
         std::set<PositionedID> seen;
@@ -2378,31 +2384,31 @@ PositionedIDVec NavDataCache::airwayWaypts(int id)
         PositionedIDDeque linearAirway;
         PositionedID firstId = rawEdges.front().first,
             lastId = rawEdges.front().second;
-        
+
         // first edge is trivial
         linearAirway.push_back(firstId);
         linearAirway.push_back(lastId);
         seen.insert(firstId);
         seen.insert(lastId);
         rawEdges.pop_front();
-        
+
         while (!rawEdges.empty()) {
             Edge e = rawEdges.front();
             rawEdges.pop_front();
-            
+
             bool seenFirst = (seen.find(e.first) != seen.end());
             bool seenSecond = (seen.find(e.second) != seen.end());
-            
+
             // duplicated segment, should be impossible
             assert(!(seenFirst && seenSecond));
-            
+
             if (!seenFirst && !seenSecond) {
                 // push back to try later on
                 nextDeque.push_back(e);
                 if (rawEdges.empty()) {
                     rawEdges = nextDeque;
                     nextDeque.clear();
-                    
+
                     if (!didAddEdge) {
                         // we have a disjoint, need to start a new section
                         // break out of the inner while loop so the outer
@@ -2411,10 +2417,10 @@ PositionedIDVec NavDataCache::airwayWaypts(int id)
                     }
                     didAddEdge = false;
                 }
-                
+
                 continue;
             }
-            
+
             // we have an exterior edge, grow our current linear piece
             if (seenFirst && (e.first == firstId)) {
                 linearAirway.push_front(e.second);
@@ -2434,20 +2440,20 @@ PositionedIDVec NavDataCache::airwayWaypts(int id)
                 seen.insert(e.first);
             }
             didAddEdge = true;
-            
+
             if (rawEdges.empty()) {
                 rawEdges = nextDeque;
                 nextDeque.clear();
             }
         }
-        
+
         if (!result.empty())
             result.push_back(0);
         result.insert(result.end(), linearAirway.begin(), linearAirway.end());
     } // outer loop
-    
+
     SG_LOG(SG_AUTOPILOT, SG_WARN, "Airway:" << id);
-    for (int i=0; i<result.size(); ++i) {
+    for (unsigned int i=0; i<result.size(); ++i) {
         if (result.at(i) == 0) {
             SG_LOG(SG_AUTOPILOT, SG_WARN, i << " <break>");
         } else {
@@ -2455,7 +2461,7 @@ PositionedIDVec NavDataCache::airwayWaypts(int id)
 
         }
     }
-    
+
     return result;
 }
 
@@ -2528,7 +2534,7 @@ public:
                 break;
             } else if (err == SQLITE_ROW) {
                 PositionedID r = sqlite3_column_int64(query, 0);
-                SGGuard<SGMutex> g(lock);
+                std::lock_guard<std::mutex> g(lock);
                 results.push_back(r);
             } else if (err == SQLITE_BUSY) {
                 // sleep a tiny amount
@@ -2539,11 +2545,11 @@ public:
             }
         }
 
-        SGGuard<SGMutex> g(lock);
+        std::lock_guard<std::mutex> g(lock);
         isComplete = true;
     }
 
-    SGMutex lock;
+    std::mutex lock;
     sqlite3* db;
     sqlite3_stmt_ptr query;
     PositionedIDVec results;
@@ -2575,7 +2581,7 @@ NavDataCache::ThreadedGUISearch::ThreadedGUISearch(const std::string& term, bool
 NavDataCache::ThreadedGUISearch::~ThreadedGUISearch()
 {
     {
-        SGGuard<SGMutex> g(d->lock);
+        std::lock_guard<std::mutex> g(d->lock);
         d->quit = true;
     }
 
@@ -2588,7 +2594,7 @@ PositionedIDVec NavDataCache::ThreadedGUISearch::results() const
 {
     PositionedIDVec r;
     {
-        SGGuard<SGMutex> g(d->lock);
+        std::lock_guard<std::mutex> g(d->lock);
         r = std::move(d->results);
     }
     return r;
@@ -2596,7 +2602,7 @@ PositionedIDVec NavDataCache::ThreadedGUISearch::results() const
 
 bool NavDataCache::ThreadedGUISearch::isComplete() const
 {
-    SGGuard<SGMutex> g(d->lock);
+    std::lock_guard<std::mutex> g(d->lock);
     return d->isComplete;
 }
 
