@@ -239,6 +239,10 @@ TurnInfo turnCenterAndAngleFromExit(const SGGeod& pt, double outHeadingDeg,
     return r;
 }
 
+class WayptData;
+using WayptDataVec = std::vector<WayptData>;
+using WpDataIt =  WayptDataVec::iterator;
+
 class WayptData
 {
 public:
@@ -272,11 +276,6 @@ public:
     } else {
       pos = wpt->position();
       posValid = true;
-      
-      if (ty == "hold") {
-        legCourseTrue = wpt->headingRadialDeg() + magVarFor(pos);
-        legCourseValid = true;
-      }
     } // of static waypt
   }
 
@@ -290,9 +289,10 @@ public:
   }
 
   // compute leg courses for all static legs (both ends are fixed)
-  void initPass1(const WayptData& previous, WayptData* next)
+  void initPass1(const WayptData* previous, WayptData* next)
   {
-    if (wpt->type() == "vectors") {
+    const auto ty = wpt->type();
+    if ((ty == "vectors") || (ty == "discontinuity")) {
       // relying on the fact vectors will be followed by a static fix/wpt
       if (next && next->posValid) {
         posValid = true;
@@ -300,69 +300,83 @@ public:
       }
     }
 
-    if (wpt->type() == "via") {
+    const bool previousPosValid = previous && previous->posValid;
+    if (ty == "via") {
         // even though both ends may be known, we don't
         // want to compute a leg course for a VIA
-    } else if (posValid && !legCourseValid && previous.posValid) {
+    } else if (posValid && !legCourseValid && previousPosValid) {
+      const auto prevWP = previous->wpt;
         // check for duplicate points now
-      if (previous.wpt->matches(wpt)) {
+      if (prevWP->matches(wpt)) {
           skipped = true;
       }
 
       // we can compute leg course now
-        if (previous.wpt->type() == "runway") {
+        if (prevWP->type() == "runway") {
             // use the runway departure end pos
-            FGRunway* rwy = static_cast<RunwayWaypt*>(previous.wpt.get())->runway();
+            FGRunway* rwy = static_cast<RunwayWaypt*>(prevWP.get())->runway();
             legCourseTrue = SGGeodesy::courseDeg(rwy->end(), pos);
-        } else if (wpt->type() != "runway") {
+        } else if (ty == "discontinuity") {
+            // we want to keep the path across DISCONs sharp (no turn entry/exit)
+            // so we keep the leg course invalid, even though we actually
+            // could compute it
+        } else if (ty != "runway") {
             // need to wait to compute runway leg course
-            legCourseTrue = SGGeodesy::courseDeg(previous.pos, pos);
+            legCourseTrue = SGGeodesy::courseDeg(previous->pos, pos);
             legCourseValid = true;
         }
     } // of not a VIA
   }
 
-  void computeLegCourse(const WayptData& previous, double radiusM)
+  void computeLegCourse(const WayptData* previous, double radiusM)
   {
-      if (legCourseValid) {
-          return;
-      }
-
-      if ((wpt->type() == "hold") ||
-              (wpt->type() == "discontinuity") ||
-              (wpt->type()  == "via"))
-      {
-          // do nothing, we can't compute a valid leg course for these types
-          // we'll generate sharp turns in the path but that's no problem.
-      } else if (wpt->type() == "runway") {
-          FGRunway* rwy = static_cast<RunwayWaypt*>(wpt.get())->runway();
-          flyOver = true;
-
-          TurnInfo ti = turnCenterAndAngleFromExit(rwy->threshold(),
-                                                   rwy->headingDeg(),
-                                                   radiusM, previous.pos);
-          if (ti.valid) {
-              legCourseTrue = ti.inboundCourseDeg;
-              turnEntryAngle = ti.turnAngleDeg;
-              turnEntryCenter = ti.turnCenter;
-              turnRadius = radiusM;
-              hasEntry = true;
-              turnEntryPos = pointOnEntryTurnFromHeading(ti.inboundCourseDeg);
-          } else {
-              // couldn't compute entry, never mind
-              legCourseTrue = SGGeodesy::courseDeg(previous.pos, rwy->threshold());
-          }
-          legCourseValid = true;
+    if (legCourseValid) {
+      return;
+    }
+    
+    if ((wpt->type() == "via") || (wpt->type() == "discontinuty"))
+    {
+      // do nothing, we can't compute a valid leg course for these types
+      // we'll generate sharp turns in the path but that's no problem.
+      return;
+    }
+    
+    if (!previous || !previous->posValid) {
+      // all the cases below assume previous pas exists and has a valid position
+      SG_LOG(SG_NAVAID, SG_WARN, "RoutePath: Asked to compute leg course, but, previous is invalid");
+      return;
+    }
+    
+    if (wpt->type() == "runway") {
+      FGRunway* rwy = static_cast<RunwayWaypt*>(wpt.get())->runway();
+      flyOver = true;
+      
+      TurnInfo ti = turnCenterAndAngleFromExit(rwy->threshold(),
+                                               rwy->headingDeg(),
+                                               radiusM, previous->pos);
+      if (ti.valid) {
+        legCourseTrue = ti.inboundCourseDeg;
+        turnEntryAngle = ti.turnAngleDeg;
+        turnEntryCenter = ti.turnCenter;
+        turnRadius = radiusM;
+        hasEntry = true;
+        turnEntryPos = pointOnEntryTurnFromHeading(ti.inboundCourseDeg);
       } else {
-          if (posValid) {
-              legCourseTrue = SGGeodesy::courseDeg(previous.pos, pos);
-              legCourseValid = true;
-          } else if (isCourseConstrained()) {
-              double magVar = magVarFor(previous.pos);
-              legCourseTrue = wpt->headingRadialDeg() + magVar;
-              legCourseValid = true;
-          } // of pos not valid
-      } // of general case
+        // couldn't compute entry, never mind
+        legCourseTrue = SGGeodesy::courseDeg(previous->pos, rwy->threshold());
+      }
+      legCourseValid = true;
+      return;
+    }
+    
+    if (posValid) {
+      legCourseTrue = SGGeodesy::courseDeg(previous->pos, pos);
+      legCourseValid = true;
+    } else if (isCourseConstrained()) {
+      double magVar = magVarFor(previous->pos);
+      legCourseTrue = wpt->headingRadialDeg() + magVar;
+      legCourseValid = true;
+    } // of pos not valid
   }
 
     SGGeod pointOnEntryTurnFromHeading(double headingDeg) const
@@ -405,8 +419,12 @@ public:
               legCourseTrue = rwy->headingDeg();
               flyOver = true;
           } else {
-              // don't set legCourseValid
+              legCourseValid = true;
+              legCourseTrue = next.legCourseTrue;
               turnExitAngle = 0.0;
+              turnExitPos = pos;
+              flyOver = true;
+              return;
           }
       }
 
@@ -598,8 +616,6 @@ public:
   bool flyOver;
 };
 
-typedef std::vector<WayptData> WayptDataVec;
-
 bool isDescentWaypoint(const WayptRef& wpt)
 {
   return (wpt->flag(WPT_APPROACH) && !wpt->flag(WPT_MISS)) || wpt->flag(WPT_ARRIVAL);
@@ -616,9 +632,13 @@ public:
   void computeDynamicPosition(int index)
   {
     auto previous(previousValidWaypoint(index));
+    if ((previous == waypoints.end()) || !previous->posValid)
+    {
+      SG_LOG(SG_NAVAID, SG_WARN, "couldn't compute position for dynamic waypoint: no preceeding valid waypoint");
+      return;
+    }
+    
     WayptRef wpt = waypoints[index].wpt;
-    assert(previous != waypoints.end());
-    assert(previous->posValid);
 
     const std::string& ty(wpt->type());
     if (ty == "hdgToAlt") {
@@ -795,14 +815,13 @@ public:
   
     WayptDataVec::iterator previousValidWaypoint(unsigned int index)
     {
-        if (index == 0) {
-            return waypoints.end();
-        }
-
-        while (waypoints[--index].skipped) {
-            // waypoint zero should be unskippable, this assert verified that
-            assert(index > 0);
-        }
+        do {
+            if (index == 0) {
+                return waypoints.end();
+            }
+            
+            --index;
+        } while (waypoints.at(index).skipped || (waypoints.at(index).wpt->type() == "discontinuity"));
 
         return waypoints.begin() + index;
     }
@@ -825,7 +844,7 @@ public:
         }
 
         ++it;
-        while ((it != waypoints.end()) && it->skipped) {
+        while ((it != waypoints.end()) && (it->skipped || (it->wpt->type() == "discontinuity"))) {
             ++it;
         }
         return it;
@@ -865,15 +884,16 @@ RoutePath::~RoutePath()
 }
 
 void RoutePath::commonInit()
-{  
-  WayptDataVec::iterator it;
-  for (it = d->waypoints.begin(); it != d->waypoints.end(); ++it) {
-    it->initPass0();
+{
+  for (auto& w : d->waypoints) {
+    w.initPass0();
   }
   
   for (unsigned int i=1; i<d->waypoints.size(); ++i) {
     WayptData* nextPtr = ((i + 1) < d->waypoints.size()) ? &d->waypoints[i+1] : nullptr;
-    d->waypoints[i].initPass1(d->waypoints[i-1], nextPtr);
+    auto prev = d->previousValidWaypoint(i);
+    WayptData* prevPtr = (prev == d->waypoints.end()) ? nullptr : &(*prev);
+    d->waypoints[i].initPass1(prevPtr, nextPtr);
   }
 
   for (unsigned int i=0; i<d->waypoints.size(); ++i) {
@@ -886,14 +906,15 @@ void RoutePath::commonInit()
 
       if (i > 0) {
           auto prevIt = d->previousValidWaypoint(i);
-          assert(prevIt != d->waypoints.end());
-          d->waypoints[i].computeLegCourse(*prevIt, radiusM);
+          WayptData* prevPtr = (prevIt == d->waypoints.end()) ? nullptr : &(*prevIt);
+
+          d->waypoints[i].computeLegCourse(prevPtr, radiusM);
           d->computeDynamicPosition(i);
       }
 
       auto nextIt = d->nextValidWaypoint(i);
       if (nextIt != d->waypoints.end()) {
-          nextIt->computeLegCourse(d->waypoints[i], radiusM);
+          nextIt->computeLegCourse(&(d->waypoints[i]), radiusM);
 
           if (nextIt->legCourseValid) {
               d->waypoints[i].computeTurn(radiusM, d->constrainLegCourses, *nextIt);
@@ -937,10 +958,6 @@ SGGeodVec RoutePath::pathForIndex(int index) const
       return pathForVia(static_cast<Via*>(d->waypoints[index].wpt.get()), index);
   }
   
-  if (ty== "hold") {
-    return pathForHold((Hold*) d->waypoints[index].wpt.get());
-  }
-
     auto prevIt = d->previousValidWaypoint(index);
     if (prevIt != d->waypoints.end()) {
         prevIt->turnExitPath(r);
@@ -957,6 +974,13 @@ SGGeodVec RoutePath::pathForIndex(int index) const
     } // of have previous waypoint
 
     w.turnEntryPath(r);
+    
+    // hold is the normal leg and then the hold waypoints as well
+    if (ty== "hold") {
+        const auto h = static_cast<Hold*>(d->waypoints[index].wpt.get());
+        const auto holdPath = pathForHold(h);
+        r.insert(r.end(), holdPath.begin(), holdPath.end());
+    }
   
   if (ty == "runway") {
     // runways get an extra point, at the end. this is particularly
@@ -1003,7 +1027,6 @@ SGGeodVec RoutePath::pathForVia(Via* via, int index) const
     auto prevIt = d->previousValidWaypoint(index);
     if (prevIt == d->waypoints.end()) {
         return SGGeodVec();
-
     }
 
     WayptVec enrouteWaypoints = via->expandToWaypoints(prevIt->wpt);
@@ -1073,12 +1096,19 @@ double RoutePath::computeDistanceForIndex(int index) const
         return 0.0;
     }
 
-    if (it->wpt->type() == "via") {
+    const auto ty = it->wpt->type();
+    if (ty == "via") {
         return distanceForVia(static_cast<Via*>(it->wpt.get()), index);
     }
+    
+    if (ty == "discontinuity") {
+        return 0.0;
+    }
 
-    auto prevIt = d->previousValidWaypoint(index);
-    assert(prevIt != d->waypoints.end());
+  auto prevIt = d->previousValidWaypoint(index);
+  if (prevIt == d->waypoints.end()) {
+    return 0.0;
+  }
 
     double dist = SGGeodesy::distanceM(prevIt->turnExitPos, it->turnEntryPos);
     dist += prevIt->turnDistanceM();
@@ -1135,69 +1165,80 @@ double RoutePath::distanceBetweenIndices(int from, int to) const
 
 SGGeod RoutePath::positionForDistanceFrom(int index, double distanceM) const
 {
-    int sz = (int) d->waypoints.size();
-    if (index < 0) {
-        index = sz - 1; // map negative values to end of the route
+  int sz = (int) d->waypoints.size();
+  if (index < 0) {
+    index = sz - 1; // map negative values to end of the route
+  }
+  
+  if ((index < 0) || (index >= sz)) {
+    throw sg_range_exception("waypt index out of range",
+                             "RoutePath::positionForDistanceFrom");
+  }
+  
+  // find the actual leg we're within
+  if (distanceM < 0.0) {
+    // scan backwards
+    while ((index > 0) && (distanceM < 0.0)) {
+      // we are looking at index n, say 4, but with a negative distance.
+      // we want to look at index n-1 (so, 3), and see if this makes
+      // distance positive. We need to offset by distance from 3 -> 4,
+      // which is waypoint 4's path distance.
+      
+      // note pathDistanceM is 0 for skipped waypoints, so this works out
+      distanceM += d->waypoints[index].pathDistanceM;
+      --index;
     }
-
-    if ((index < 0) || (index >= sz)) {
-        throw sg_range_exception("waypt index out of range",
-                                 "RoutePath::positionForDistanceFrom");
-    }
-
-    // find the actual leg we're within
+    
     if (distanceM < 0.0) {
-        // scan backwards
-        while ((index > 0) && (distanceM < 0.0)) {
-            // we are looking at index n, say 4, but with a negative distance.
-            // we want to look at index n-1 (so, 3), and see if this makes
-            // distance positive. We need to offset by distance from 3 -> 4,
-            // which is waypoint 4's path distance.
-            distanceM += d->waypoints[index].pathDistanceM;
-            --index;
-        }
-
-        if (distanceM < 0.0) {
-            // still negative, return route start
-            return d->waypoints[0].pos;
-        }
-
-    } else {
-        // scan forwards
-        int nextIndex = index + 1;
-        while ((nextIndex < sz) && (d->waypoints[nextIndex].pathDistanceM < distanceM)) {
-            distanceM -= d->waypoints[nextIndex].pathDistanceM;
-            index = nextIndex++;
-        }
+      // still negative, return route start
+      return d->waypoints[0].pos;
     }
-
-    if ((index + 1) >= sz) {
-        // past route end, just return final position
-        return d->waypoints[sz - 1].pos;
+    
+  } else {
+    // scan forwards
+    int nextIndex = index + 1;
+    while ((nextIndex < sz) && (d->waypoints[nextIndex].pathDistanceM < distanceM)) {
+      distanceM -= d->waypoints[nextIndex].pathDistanceM;
+      index = nextIndex++;
     }
-
-
-    const WayptData& wpt(d->waypoints[index]);
-    const WayptData& next(d->waypoints[index+1]);
-    if (next.wpt->type() == "via") {
-        return positionAlongVia(static_cast<Via*>(next.wpt.get()), index, distanceM);
-    }
-
-    if (wpt.turnPathDistanceM > distanceM) {
-        // on the exit path of current wpt
-        return wpt.pointAlongExitPath(distanceM);
-    } else {
-        distanceM -= wpt.turnPathDistanceM;
-    }
-
-    double corePathDistance = next.pathDistanceM - next.turnPathDistanceM;
-    if (next.hasEntry && (distanceM > corePathDistance)) {
-        // on the entry path of next waypoint
-        return next.pointAlongEntryPath(distanceM - corePathDistance);
-    }
-
-    // linear between turn exit and turn entry points
-    return SGGeodesy::direct(wpt.turnExitPos, next.legCourseTrue, distanceM);
+  }
+  
+  auto nextIt = d->nextValidWaypoint(index);
+  if (nextIt == d->waypoints.end()) {
+    // past route end, just return final position
+    return d->waypoints[sz - 1].pos;
+  }
+  
+  // this is important so we start from a valid WP if we're
+  // working either side of a DISCON
+  auto curIt = d->previousValidWaypoint(nextIt);
+  if (curIt == d->waypoints.end()) {
+    SG_LOG(SG_NAVAID, SG_WARN, "Couldn't find valid preceeding waypoint " << index);
+    return nextIt->pos;
+  }
+  
+  const WayptData& wpt = *curIt;
+  const WayptData& next = *nextIt;
+  
+  if (next.wpt->type() == "via") {
+    return positionAlongVia(static_cast<Via*>(next.wpt.get()), index, distanceM);
+  }
+  
+  if (wpt.turnPathDistanceM > distanceM) {
+    // on the exit path of current wpt
+    return wpt.pointAlongExitPath(distanceM);
+  } else {
+    distanceM -= wpt.turnPathDistanceM;
+  }
+  
+  double corePathDistance = next.pathDistanceM - next.turnPathDistanceM;
+  if (next.hasEntry && (distanceM > corePathDistance)) {
+    // on the entry path of next waypoint
+    return next.pointAlongEntryPath(distanceM - corePathDistance);
+  }
+  
+  // linear between turn exit and turn entry points
+  return SGGeodesy::direct(wpt.turnExitPos, next.legCourseTrue, distanceM);
 }
 
 SGGeod RoutePath::positionAlongVia(Via* via, int previousIndex, double distanceM) const
