@@ -11,9 +11,10 @@
 #  include <sys/time.h>  // gettimeofday
 #endif
 
-#include <string.h>
+#include <algorithm>
 #include <errno.h>
 #include <stdio.h>
+#include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fstream>
@@ -37,6 +38,7 @@
 #include <simgear/nasal/cppbind/to_nasal.hxx>
 #include <simgear/nasal/cppbind/Ghost.hxx>
 #include <simgear/nasal/cppbind/NasalHash.hxx>
+#include <simgear/timing/timestamp.hxx>
 
 #include "NasalAddons.hxx"
 #include "NasalSGPath.hxx"
@@ -54,6 +56,7 @@
 #include <Main/globals.hxx>
 #include <Main/util.hxx>
 #include <Main/fg_props.hxx>
+#include <Main/sentryIntegration.hxx>
 
 using std::map;
 using std::string;
@@ -99,9 +102,9 @@ public:
     _self(self),
     _interval(interval)
   {
-    char nm[128];
+    char nm[256];
     if (c) {
-        snprintf(nm, 128, "maketimer-%s:%d", naStr_data(naGetSourceFile(c, 0)), naGetLine(c, 0));
+        snprintf(nm, 128, "maketimer-[%p]-%s:%d", (void*)this, naStr_data(naGetSourceFile(c, 0)), naGetLine(c, 0));
     }
     else {
         snprintf(nm, 128, "maketimer-%p", this);
@@ -185,6 +188,9 @@ public:
 
   bool isSingleShot() const
   { return _singleShot; }
+
+  const std::string& name() const
+  { return _name; }
 private:
   std::string _name;
   FGNasalSys* _sys;
@@ -208,37 +214,41 @@ static void f_timerObj_setSimTime(TimerObj& timer, naContext c, naRef value)
 
     timer.setSimTime(nasal::from_nasal<bool>(c, value));
 }
+////////////////////
+/// Timestamp - used to provide millisecond based timing of operations. See SGTimeStamp
+////
+/// 0.373404us to perform elapsedUSec from Nasal : Tested i7-4790K, Win64
+class TimeStampObj : public SGReferenced
+{
+public:
+    TimeStampObj(Context *c)
+    {
+        timestamp.stamp();
+    }
+
+    virtual ~TimeStampObj() = default;
+
+    void stamp()
+    {
+        timestamp.stamp();
+    }
+    double elapsedMSec()
+    {
+        return timestamp.elapsedMSec();
+    }
+    double elapsedUSec()
+    {
+        return timestamp.elapsedUSec();
+    }
+private:
+    SGTimeStamp timestamp;
+};
+
+typedef SGSharedPtr<TimeStampObj> TimeStampObjRef;
+typedef nasal::Ghost<TimeStampObjRef> NasalTimeStampObj;
 
 ///////////////////////////////////////////////////////////////////////////
 
-// Read and return file contents in a single buffer.  Note use of
-// stat() to get the file size.  This is a win32 function, believe it
-// or not. :) Note the REALLY IMPORTANT use of the "b" flag to fopen.
-// Text mode brain damage will kill us if we're trying to do bytewise
-// I/O.
-static char* readfile(const char* file, int* lenOut)
-{
-    struct stat data;
-    if(stat(file, &data) != 0) return 0;
-    FILE* f = fopen(file, "rb");
-    if(!f) return 0;
-    char* buf = new char[data.st_size];
-    *lenOut = fread(buf, 1, data.st_size, f);
-    fclose(f);
-    if(*lenOut != data.st_size) {
-        // Shouldn't happen, but warn anyway since it represents a
-        // platform bug and not a typical runtime error (missing file,
-        // etc...)
-        SG_LOG(SG_NASAL, SG_ALERT,
-               "ERROR in Nasal initialization: " <<
-               "short count returned from fread() of " << file <<
-               ".  Check your C library!");
-        delete[] buf;
-        return 0;
-    }
-    return buf;
-}
-
 FGNasalSys::FGNasalSys() :
     _inited(false)
 {
@@ -302,7 +312,9 @@ naRef FGNasalSys::callMethodWithContext(naContext ctx, naRef code, naRef self, i
         return naCallMethodCtx(ctx, code, self, argc, args, locals);
     } catch (sg_exception& e) {
         SG_LOG(SG_NASAL, SG_DEV_ALERT, "caught exception invoking nasal method:" << e.what());
-        logNasalStack(ctx);
+        string_list nasalStack;
+        logNasalStack(ctx, nasalStack);
+        flightgear::sentryReportNasalError(string{"Exception invoking nasal method:"} + e.what(), nasalStack);
         return naNil();
     }
 }
@@ -313,7 +325,7 @@ FGNasalSys::~FGNasalSys()
         SG_LOG(SG_GENERAL, SG_ALERT, "Nasal was not shutdown");
     }
     sglog().removeCallback(_log.get());
-    
+
     nasalSys = nullptr;
 }
 
@@ -327,7 +339,7 @@ bool FGNasalSys::parseAndRunWithOutput(const std::string& source, std::string& o
         return false;
     }
     naRef result = callWithContext(ctx, code, 0, 0, naNil());
-    
+
     // if there was a result value, try to convert it to a string
     // value.
     if (!naIsNil(result)) {
@@ -336,7 +348,7 @@ bool FGNasalSys::parseAndRunWithOutput(const std::string& source, std::string& o
             output = naStr_data(s);
         }
     }
-    
+
     naFreeContext(ctx);
     return true;
 }
@@ -550,12 +562,20 @@ static naRef f_makeTimer(naContext c, naRef me, int argc, naRef* args)
   } else if ((argc == 3) && naIsFunc(args[2])) {
     self = args[1];
     func = args[2];
+  } else {
+    naRuntimeError(c, "bad function/self arguments to maketimer");
   }
 
   TimerObj* timerObj = new TimerObj(c, nasalSys, func, self, args[0].num);
   return nasal::to_nasal(c, timerObj);
 }
 
+static naRef f_maketimeStamp(naContext c, naRef me, int argc, naRef* args)
+{
+    TimeStampObj* timeStampObj = new TimeStampObj(c);
+    return nasal::to_nasal(c, timeStampObj);
+}
+
 // setlistener(func, property, bool) extension function.  Falls through to
 // FGNasalSys::setListener().  See there for docs.
 static naRef f_setlistener(naContext c, naRef me, int argc, naRef* args)
@@ -758,13 +778,65 @@ static naRef f_open(naContext c, naRef me, int argc, naRef* args)
     std::wstring wmodestr = simgear::strutils::convertUtf8ToWString(modestr);
     f = _wfopen(fp.c_str(), wmodestr.c_str());
 #else
-    std::string fp = filename.local8BitStr();
+    std::string fp = filename.utf8Str();
     f = fopen(fp.c_str(), modestr);
 #endif
     if(!f) naRuntimeError(c, strerror(errno));
     return naIOGhost(c, f);
 }
 
+static naRef ftype(naContext ctx, const SGPath& f)
+{
+    const char* t = "unk";
+    if (f.isFile()) t = "reg";
+    else if(f.isDir()) t = "dir";
+    return naStr_fromdata(naNewString(ctx), t, strlen(t));
+}
+
+// io.stat with UTF-8 path support, replaces the default one in
+// Nasal iolib.c which does not hsupport UTF-8 paths
+static naRef f_custom_stat(naContext ctx, naRef me, int argc, naRef* args)
+{
+    naRef pathArg = argc > 0 ? naStringValue(ctx, args[0]) : naNil();
+    if (!naIsString(pathArg))
+        naRuntimeError(ctx, "bad argument to stat()");
+    
+    const auto path = SGPath::fromUtf8(naStr_data(pathArg));
+    if (!path.exists()) {
+        return naNil();
+    }
+    
+    const SGPath filename = fgValidatePath(path, false );
+    if (filename.isNull()) {
+        SG_LOG(SG_NASAL, SG_ALERT, "stat(): reading '" <<
+        naStr_data(pathArg) << "' denied (unauthorized directory - authorization"
+        " no longer follows symlinks; to authorize reading additional "
+        "directories, pass them to --allow-nasal-read)");
+        naRuntimeError(ctx, "stat(): access denied (unauthorized directory)");
+        return naNil();
+    }
+   
+    naRef result = naNewVector(ctx);
+    naVec_setsize(ctx, result, 12);
+    
+    // every use in fgdata/Nasal only uses stat() to test existence of
+    // files, not to check any of this information.
+    int n = 0;
+    naVec_set(result, n++, naNum(0)); // device
+    naVec_set(result, n++, naNum(0)); // inode
+    naVec_set(result, n++, naNum(0)); // mode
+    naVec_set(result, n++, naNum(0)); // nlink
+    naVec_set(result, n++, naNum(0)); // uid
+    naVec_set(result, n++, naNum(0)); // guid
+    naVec_set(result, n++, naNum(0)); // rdev
+    naVec_set(result, n++, naNum(filename.sizeInBytes())); // size
+    naVec_set(result, n++, naNum(0)); // atime
+    naVec_set(result, n++, naNum(0)); // mtime
+    naVec_set(result, n++, naNum(0)); // ctime
+    naVec_set(result, n++, ftype(ctx, filename));
+    return result;
+}
+
 // Parse XML file.
 //     parsexml(<path> [, <start-tag> [, <end-tag> [, <data> [, <pi>]]]]);
 //
@@ -878,6 +950,7 @@ static struct { const char* name; naCFunction func; } funcs[] = {
     { "parse_markdown", f_parse_markdown },
     { "md5", f_md5 },
     { "systime", f_systime },
+    { "maketimestamp", f_maketimeStamp},
     { 0, 0 }
 };
 
@@ -886,6 +959,17 @@ naRef FGNasalSys::cmdArgGhost()
     return propNodeGhost(_cmdArg);
 }
 
+void FGNasalSys::initLogLevelConstants()
+{
+    hashset(_globals, "LOG_BULK", naNum(SG_BULK));
+    hashset(_globals, "LOG_WARN", naNum(SG_WARN));
+    hashset(_globals, "LOG_DEBUG", naNum(SG_DEBUG));
+    hashset(_globals, "LOG_INFO", naNum(SG_INFO));
+    hashset(_globals, "LOG_ALERT", naNum(SG_ALERT));
+    hashset(_globals, "DEV_WARN", naNum(SG_DEV_WARN));
+    hashset(_globals, "DEV_ALERT", naNum(SG_DEV_ALERT));
+}
+
 void FGNasalSys::setCmdArg(SGPropertyNode* aNode)
 {
     _cmdArg = aNode;
@@ -914,12 +998,15 @@ void FGNasalSys::init()
     hashset(_globals, "thread", naInit_thread(_context));
     hashset(_globals, "utf8", naInit_utf8(_context));
 
+    initLogLevelConstants();
+    
     // Add our custom extension functions:
     for(i=0; funcs[i].name; i++)
         hashset(_globals, funcs[i].name,
                 naNewFunc(_context, naNewCCode(_context, funcs[i].func)));
     nasal::Hash io_module = getGlobals().get<nasal::Hash>("io");
     io_module.set("open", f_open);
+    io_module.set("stat", f_custom_stat);
 
     // And our SGPropertyNode wrapper
     hashset(_globals, "props", genPropsModule());
@@ -946,6 +1033,12 @@ void FGNasalSys::init()
       .member("simulatedTime", &TimerObj::isSimTime, &f_timerObj_setSimTime)
       .member("isRunning", &TimerObj::isRunning);
 
+    NasalTimeStampObj::init("TimeStamp")
+        .method("stamp", &TimeStampObj::stamp)
+        .method("elapsedMSec", &TimeStampObj::elapsedMSec)
+        .method("elapsedUSec", &TimeStampObj::elapsedUSec)
+        ;
+
     flightgear::addons::initAddonClassesForNasal(_globals, _context);
 
     // Now load the various source files in the Nasal directory
@@ -1002,7 +1095,7 @@ void FGNasalSys::shutdown()
         delete t;
     }
     _nasalTimers.clear();
-    
+
     naClearSaved();
 
     _string = naNil(); // will be freed by _context
@@ -1015,15 +1108,19 @@ void FGNasalSys::shutdown()
     _globals = naNil();
 
     naGC();
-    
+
     // Destroy all queued ghosts : important to ensure persistent timers are
     // destroyed now.
     nasal::ghostProcessDestroyList();
-    
+
     if (!_persistentTimers.empty()) {
         SG_LOG(SG_NASAL, SG_DEV_WARN, "Extant persistent timer count:" << _persistentTimers.size());
+
+        for (auto pt : _persistentTimers) {
+            SG_LOG(SG_NASAL, SG_DEV_WARN, "Extant:" << pt << " : " << pt->name());
+        }
     }
-    
+
     _inited = false;
 }
 
@@ -1107,7 +1204,7 @@ void FGNasalSys::addModule(string moduleName, simgear::PathList scripts)
 {
     if (! scripts.empty())
     {
-        SGPropertyNode* nasal = globals->get_props()->getNode("nasal");
+        SGPropertyNode* nasal = globals->get_props()->getNode("nasal", true);
         SGPropertyNode* module_node = nasal->getChild(moduleName,0,true);
         for (unsigned int i=0; i<scripts.size(); ++i) {
             SGPropertyNode* pFileNode = module_node->getChild("file",i,true);
@@ -1116,8 +1213,14 @@ void FGNasalSys::addModule(string moduleName, simgear::PathList scripts)
         if (!module_node->hasChild("enabled",0))
         {
             SGPropertyNode* node = module_node->getChild("enabled",0,true);
-            node->setBoolValue(true);
-            node->setAttribute(SGPropertyNode::USERARCHIVE,true);
+            node->setBoolValue(false);
+            node->setAttribute(SGPropertyNode::USERARCHIVE,false);
+            SG_LOG(SG_NASAL, SG_ALERT, "Nasal module " <<
+            moduleName <<
+            " present in FGDATA/Nasal but not configured in defaults.xml. " <<
+            " Please add an entry to defaults.xml, and set "
+            << node->getPath() <<
+            "=true to load the module on-demand at runtime when required.");
         }
     }
 }
@@ -1206,23 +1309,34 @@ void FGNasalSys::loadPropertyScripts(SGPropertyNode* n)
 // Logs a runtime error, with stack trace, to the FlightGear log stream
 void FGNasalSys::logError(naContext context)
 {
-    SG_LOG(SG_NASAL, SG_ALERT, "Nasal runtime error: " << naGetError(context));
-    logNasalStack(context);
+    string errorMessage = naGetError(context);
+    SG_LOG(SG_NASAL, SG_ALERT, "Nasal runtime error: " << errorMessage);
+    
+    string_list nasalStack;
+    logNasalStack(context, nasalStack);
+    flightgear::sentryReportNasalError(errorMessage, nasalStack);
 }
 
-void FGNasalSys::logNasalStack(naContext context)
+void FGNasalSys::logNasalStack(naContext context, string_list& stack)
 {
     const int stack_depth = naStackDepth(context);
     if (stack_depth < 1)
       return;
 
+    stack.push_back(string{naStr_data(naGetSourceFile(context, 0))} +
+                    ", line " + std::to_string(naGetLine(context, 0)));
+    
     SG_LOG(SG_NASAL, SG_ALERT,
            "  at " << naStr_data(naGetSourceFile(context, 0)) <<
            ", line " << naGetLine(context, 0));
+    
     for(int i=1; i<stack_depth; i++) {
         SG_LOG(SG_NASAL, SG_ALERT,
                "  called from: " << naStr_data(naGetSourceFile(context, i)) <<
                ", line " << naGetLine(context, i));
+        
+        stack.push_back(string{naStr_data(naGetSourceFile(context, i))} +
+                        ", line " + std::to_string(naGetLine(context, i)));
     }
 }
 
@@ -1231,19 +1345,21 @@ void FGNasalSys::logNasalStack(naContext context)
 // name.
 bool FGNasalSys::loadModule(SGPath file, const char* module)
 {
-    int len = 0;
-    std::string pdata = file.local8BitStr();
-    char* buf = readfile(pdata.c_str(), &len);
-    if(!buf) {
-        SG_LOG(SG_NASAL, SG_ALERT,
-               "Nasal error: could not read script file " << file
-               << " into module " << module);
+    if (!file.exists()) {
+        SG_LOG(SG_NASAL, SG_ALERT, "Cannot load module, missing file:" << file);
         return false;
     }
 
-    bool ok = createModule(module, pdata.c_str(), buf, len);
-    delete[] buf;
-    return ok;
+    sg_ifstream file_in(file);
+    string buf;
+    while (!file_in.eof()) {
+        char bytes[8192];
+        file_in.read(bytes, 8192);
+        buf.append(bytes, file_in.gcount());
+    }
+    file_in.close();
+    auto pathStr = file.utf8Str();
+    return createModule(module, pathStr.c_str(), buf.data(), buf.length());
 }
 
 // Parse and run.  Save the local variables namespace, as it will
@@ -1320,7 +1436,7 @@ naRef FGNasalSys::parse(naContext ctx, const char* filename,
             " in "<< filename <<", line " << errLine;
         errors = errorMessageStream.str();
         SG_LOG(SG_NASAL, SG_ALERT, errors);
-               
+
         return naNil();
     }
 
@@ -1483,9 +1599,13 @@ naRef FGNasalSys::setListener(naContext c, int argc, naRef* args)
         return naNil();
     }
 
-    if(node->isTied())
-        SG_LOG(SG_NASAL, SG_DEBUG, "Attaching listener to tied property " <<
-                node->getPath());
+    if (node->isTied()) {
+        const auto isSafe = node->getAttribute(SGPropertyNode::LISTENER_SAFE);
+        if (!isSafe) {
+            SG_LOG(SG_NASAL, SG_DEV_ALERT, "ERROR: Cannot add listener to tied property " <<
+                     node->getPath());
+        }
+    }
 
     naRef code = argc > 1 ? args[1] : naNil();
     if(!(naIsCode(code) || naIsCCode(code) || naIsFunc(code))) {
@@ -1493,8 +1613,8 @@ naRef FGNasalSys::setListener(naContext c, int argc, naRef* args)
         return naNil();
     }
 
-    int init = argc > 2 && naIsNum(args[2]) ? int(args[2].num) : 0;
-    int type = argc > 3 && naIsNum(args[3]) ? int(args[3].num) : 1;
+    int init = argc > 2 && naIsNum(args[2]) ? int(args[2].num) : 0; // do not trigger when created
+    int type = argc > 3 && naIsNum(args[3]) ? int(args[3].num) : 1; // trigger will always be triggered when the property is written
     FGNasalListener *nl = new FGNasalListener(node, code, this,
             gcSave(code), _listenerId, init, type);
 
@@ -1570,6 +1690,11 @@ void FGNasalSys::removePersistentTimer(TimerObj* obj)
     _persistentTimers.erase(it);
 }
 
+// Register the subsystem.
+SGSubsystemMgr::Registrant<FGNasalSys> registrantFGNasalSys(
+    SGSubsystemMgr::INIT);
+
+
 //////////////////////////////////////////////////////////////////////////
 // FGNasalListener class.
 
