@@ -45,12 +45,12 @@
 #include <cstring>
 #include <iostream>
 #include <fstream>
+#include <mutex>
 
 
 #include <string>
 #include <vector>
 #include <algorithm>
-#include <boost/foreach.hpp>
 
 #include <simgear/compiler.h>
 #include <simgear/misc/sg_path.hxx>
@@ -59,11 +59,10 @@
 #include <simgear/props/props.hxx>
 #include <simgear/structure/subsystem_mgr.hxx>
 #include <simgear/structure/exception.hxx>
+#include <simgear/threads/SGThread.hxx>
 #include <simgear/timing/sg_time.hxx>
 
 #include <simgear/xml/easyxml.hxx>
-#include <simgear/threads/SGThread.hxx>
-#include <simgear/threads/SGGuard.hxx>
 #include <simgear/scene/tsync/terrasync.hxx>
 
 #include <AIModel/AIAircraft.hxx>
@@ -124,20 +123,20 @@ public:
 
   bool isFinished() const
   {
-    SGGuard<SGMutex> g(_lock);
+    std::lock_guard<std::mutex> g(_lock);
     return _isFinished;
   }
 
   virtual void run()
   {
-      BOOST_FOREACH(SGPath p, _trafficDirPaths) {
+      for (const auto& p : _trafficDirPaths) {
           parseTrafficDir(p);
           if (_cancelThread) {
               return;
           }
       }
 
-    SGGuard<SGMutex> g(_lock);
+    std::lock_guard<std::mutex> g(_lock);
     _isFinished = true;
   }
 
@@ -378,11 +377,11 @@ private:
         simgear::Dir trafficDir(path);
         simgear::PathList d = trafficDir.children(simgear::Dir::TYPE_DIR | simgear::Dir::NO_DOT_OR_DOTDOT);
 
-        BOOST_FOREACH(SGPath p, d) {
+        for (const auto& p : d) {
             simgear::Dir d2(p);
             SG_LOG(SG_AI, SG_DEBUG, "parsing traffic in:" << p);
             simgear::PathList trafficFiles = d2.children(simgear::Dir::TYPE_FILE, ".xml");
-            BOOST_FOREACH(SGPath xml, trafficFiles) {
+            for (const auto& xml : trafficFiles) {
                 readXML(xml, *this);
                 if (_cancelThread) {
                     return;
@@ -394,7 +393,7 @@ private:
     }
 
   FGTrafficManager* _trafficManager;
-  mutable SGMutex _lock;
+  mutable std::mutex _lock;
   bool _isFinished;
   bool _cancelThread;
   simgear::PathList _trafficDirPaths;
@@ -427,7 +426,9 @@ FGTrafficManager::FGTrafficManager() :
   enabled("/sim/traffic-manager/enabled"),
   aiEnabled("/sim/ai/enabled"),
   realWxEnabled("/environment/realwx/enabled"),
-  metarValid("/environment/metar/valid")
+  metarValid("/environment/metar/valid"),
+  active("/sim/traffic-manager/active"),
+  aiDataUpdateNow("/sim/terrasync/ai-data-update-now")
 {
 }
 
@@ -442,6 +443,7 @@ void FGTrafficManager::shutdown()
       if (doingInit) {
         scheduleParser.reset();
         doingInit = false;
+        active = false;
       }
 
       return;
@@ -478,7 +480,7 @@ void FGTrafficManager::shutdown()
         }
     }
 
-    BOOST_FOREACH(FGAISchedule* acft, scheduledAircraft) {
+    for (auto acft : scheduledAircraft) {
         if (saveData) {
             cachefile << acft->getRegistration() << " "
                 << acft->getRunCount() << " "
@@ -497,6 +499,30 @@ void FGTrafficManager::shutdown()
     doingInit = false;
     inited = false;
     trafficSyncRequested = false;
+    active = false;
+}
+
+bool FGTrafficManager::doDataSync()
+{
+    auto terraSync = globals->get_subsystem<simgear::SGTerraSync>();
+    bool doDataSync = fgGetBool("/sim/terrasync/ai-data-enabled");
+    if (doDataSync && terraSync) {
+        if (!trafficSyncRequested) {
+            SG_LOG(SG_AI, SG_INFO, "Sync of AI traffic via TerraSync enabled");
+            terraSync->scheduleDataDir("AI/Traffic");
+            trafficSyncRequested = true;
+        }
+
+        if (terraSync->isDataDirPending("AI/Traffic")) {
+            return false; // remain in the init state
+        }
+
+        trafficSyncRequested = false;
+        return true;
+
+        SG_LOG(SG_AI, SG_INFO, "Traffic files sync complete");
+    }
+    return true;
 }
 
 void FGTrafficManager::init()
@@ -512,21 +538,9 @@ void FGTrafficManager::init()
       return;
 
     assert(!doingInit);
-    simgear::SGTerraSync* terraSync = static_cast<simgear::SGTerraSync*>(globals->get_subsystem("terrasync"));
-    bool doDataSync = fgGetBool("/sim/terrasync/ai-data-enabled");
-    if (doDataSync && terraSync) {
-        if (!trafficSyncRequested) {
-            SG_LOG(SG_AI, SG_INFO, "Sync of AI traffic via TerraSync enabled");
-            terraSync->scheduleDataDir("AI/Traffic");
-            trafficSyncRequested = true;
-        }
 
-        if (terraSync->isDataDirPending("AI/Traffic")) {
-            return; // remain in the init state
-        }
-
-        SG_LOG(SG_AI, SG_INFO, "Traffic files sync complete");
-    }
+    if (!doDataSync())
+        return; // remain in the init state whilst updating
 
     doingInit = true;
     if (string(fgGetString("/sim/traffic-manager/datafile")).empty()) {
@@ -575,6 +589,7 @@ void FGTrafficManager::init()
         }
         //exit(1);
     }
+    active = true;
 }
 
 
@@ -586,7 +601,7 @@ void FGTrafficManager::finishInit()
     loadHeuristics();
     PerformanceDB* perfDB = globals->get_subsystem<PerformanceDB>();
     // Do sorting and scoring separately, to take advantage of the "homeport" variable
-    BOOST_FOREACH(FGAISchedule* schedule, scheduledAircraft) {
+    for (auto schedule : scheduledAircraft) {
         schedule->setScore();
         if (!perfDB->havePerformanceDataForAircraftType(schedule->getAircraft())) {
             SG_LOG(SG_AI, SG_DEV_WARN, "AI-Traffic: schedule aircraft missing performance data:" << schedule->getAircraft());
@@ -600,6 +615,7 @@ void FGTrafficManager::finishInit()
 
     doingInit = false;
     inited = true;
+    active = true;
 }
 
 void FGTrafficManager::loadHeuristics()
@@ -697,6 +713,11 @@ void FGTrafficManager::update(double dt)
     if (!metarReady(dt))
         return;
 
+    if (aiDataUpdateNow)
+    {
+        aiDataUpdateNow = false;
+        shutdown();
+    }
     if (!aiEnabled)
     {
         // traffic depends on AI module
@@ -914,3 +935,10 @@ void FGTrafficManager::Tokenize(const string& str,
         pos = str.find_first_of(delimiters, lastPos);
     }
 }
+
+
+// Register the subsystem.
+SGSubsystemMgr::Registrant<FGTrafficManager> registrantFGTrafficManager(
+    SGSubsystemMgr::POST_FDM,
+    {{"terrasync", SGSubsystemMgr::Dependency::HARD},
+     {"PerformanceDB", SGSubsystemMgr::Dependency::HARD}});
