@@ -61,6 +61,10 @@
 #include <QCoreApplication>
 #endif
 
+#if defined(SG_MAC)
+#  include <GUI/CocoaHelpers.h>
+#endif
+
 // Static linking of OSG needs special macros
 #ifdef OSG_LIBRARY_STATIC
 #include <osgDB/Registry>
@@ -357,7 +361,6 @@ void fgOSInit(int* argc, char** argv)
 #if defined(HAVE_QT)
     global_usingGraphicsWindowQt = fgGetBool("/sim/rendering/graphics-window-qt", false);
     if (global_usingGraphicsWindowQt) {
-        flightgear::initApp(*argc, argv);
         SG_LOG(SG_GL, SG_INFO, "Using Qt implementation of GraphicsWindow");
         flightgear::initQtWindowingSystem();
     } else {
@@ -366,6 +369,10 @@ void fgOSInit(int* argc, char** argv)
         SG_LOG(SG_GL, SG_INFO, "Using stock OSG implementation of GraphicsWindow");
     }
 #endif
+#if defined(SG_MAC)
+    cocoaRegisterTerminateHandler();
+#endif
+    
     globals->get_renderer()->init();
     WindowSystemAdapter::setWSA(new WindowSystemAdapter);
 }
