@@ -45,6 +45,7 @@
 #include <simgear/debug/logstream.hxx>
 #include <simgear/math/sg_random.h>
 #include <simgear/misc/sg_path.hxx>
+#include <simgear/scene/util/SGReaderWriterOptions.hxx>
 
 #include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
@@ -56,8 +57,11 @@
 
 #include <sstream>
 
+
 static const char* LICENSE_URL_TEXT = "Licensed under the GNU GPL. See http://www.flightgear.org for more information";
 
+using namespace simgear;
+
 class SplashScreenUpdateCallback : public osg::NodeCallback {
 public:
     virtual void operator()(osg::Node* node, osg::NodeVisitor* nv)
@@ -81,13 +85,20 @@ SplashScreen::~SplashScreen()
 
 void SplashScreen::createNodes()
 {
+    std::string splashImage = selectSplashImage();
 #if OSG_VERSION_LESS_THAN(3,4,0)
-    _splashImage = osgDB::readImageFile(selectSplashImage());
+    _splashImage = osgDB::readImageFile(splashImage);
 #else
-    _splashImage = osgDB::readRefImageFile(selectSplashImage());
+    osg::ref_ptr<SGReaderWriterOptions> staticOptions = SGReaderWriterOptions::copyOrCreate(osgDB::Registry::instance()->getOptions());
+    staticOptions->setLoadOriginHint(SGReaderWriterOptions::LoadOriginHint::ORIGIN_SPLASH_SCREEN);
+    _splashImage = osgDB::readRefImageFile(splashImage, staticOptions);
 #endif
 
-    int width = _splashImage->s();
+    if (!_splashImage){
+        SG_LOG(SG_VIEW, SG_INFO, "Splash Image " << splashImage << " failed to load");
+        return;
+    }
+    int width               = _splashImage->s();
     int height = _splashImage->t();
     _splashImageAspectRatio = static_cast<double>(width) / height;
 
@@ -195,6 +206,21 @@ void SplashScreen::createNodes()
               nullptr, -1.0, osg::Vec4(1.0, 0.0, 0.0, 1.0));
     }
 
+#if defined(ENABLE_COMPOSITOR)
+    auto compositorText = globals->get_locale()->getLocalizedString("compositor-enabled", "sys", "Compositor");
+    addText(geode, osg::Vec2(0.5f, 0.5f), 0.03,
+            compositorText,
+            osgText::Text::CENTER_CENTER,
+            nullptr, -1.0, osg::Vec4(1.0, 0.0, 0.0, 1.0));
+
+    std::string compositorWarningText = "The compositor is still in development, please expect some visual bugs";
+    addText(geode, osg::Vec2(0.5, 0.55), 0.03,
+              compositorWarningText,
+              osgText::Text::CENTER_CENTER,
+              nullptr, -1.0, osg::Vec4(1.0, 0.0, 0.0, 1.0));
+#endif
+
+
     ///////////
 
     geometry = new osg::Geometry;
@@ -221,16 +247,15 @@ void SplashScreen::createNodes()
     _splashQuadCamera->setReferenceFrame(osg::Transform::ABSOLUTE_RF);
     _splashQuadCamera->setViewMatrix(osg::Matrix::identity());
     _splashQuadCamera->setProjectionMatrixAsOrtho2D(0.0, 1.0, 0.0, 1.0);
-    _splashQuadCamera->setClearMask( 0 );
     _splashQuadCamera->setAllowEventFocus(false);
     _splashQuadCamera->setCullingActive(false);
-    _splashQuadCamera->setRenderOrder(osg::Camera::NESTED_RENDER, 20000);
+    _splashQuadCamera->setRenderOrder(osg::Camera::NESTED_RENDER);
     
     stateSet = _splashQuadCamera->getOrCreateStateSet();
     stateSet->setMode(GL_BLEND, osg::StateAttribute::ON);
     stateSet->setAttribute(new osg::BlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA), osg::StateAttribute::ON);
     stateSet->setMode(GL_LIGHTING, osg::StateAttribute::OFF);
-    stateSet->setRenderBinDetails(1, "RenderBin");
+    stateSet->setRenderBinDetails(1000, "RenderBin");
     stateSet->setMode(GL_DEPTH_TEST, osg::StateAttribute::OFF);
     
     geometry = osg::createTexturedQuadGeometry(osg::Vec3(0.0, 0.0, 0.0),
@@ -285,12 +310,16 @@ void SplashScreen::setupLogoImage()
         return;
     }
 
-#if OSG_VERSION_LESS_THAN(3,4,0)
-    _logoImage = osgDB::readImageFile(logoPath.utf8Str());
+    osg::ref_ptr<simgear::SGReaderWriterOptions> staticOptions = simgear::SGReaderWriterOptions::copyOrCreate(osgDB::Registry::instance()->getOptions());
+    staticOptions->setLoadOriginHint(simgear::SGReaderWriterOptions::LoadOriginHint::ORIGIN_SPLASH_SCREEN);
+
+#if OSG_VERSION_LESS_THAN(3, 4, 0)
+    _logoImage = osgDB::readImageFile(logoPath.utf8Str(), staticOptions);
 #else
-    _logoImage = osgDB::readRefImageFile(logoPath.utf8Str());
+    _logoImage = osgDB::readRefImageFile(logoPath.utf8Str(), staticOptions);
 #endif
     if (!_logoImage) {
+        SG_LOG(SG_VIEW, SG_INFO, "Splash logo image " << logoPath << " failed to load");
         return;
     }
 
@@ -665,17 +694,17 @@ void fgSplashProgress( const char *identifier, unsigned int percent )
         if (kbytesPending > 0) {
             if (kbytesPending > 1024) {
                 int mBytesPending = kbytesPending >> 10;
-                oss << " " << mBytesPending << "Mb";
+                oss << " " << mBytesPending << "MB";
             } else {
-                oss << " " << kbytesPending << "Kb";
+                oss << " " << kbytesPending << "KB";
             }
         }
         if (kbytesPerSec > 0) {
             if (kbytesPerSec > 100) {
                 double mbytesPerSec = kbytesPerSec / 1024.0;
-                oss << " - " << std::fixed << std::setprecision(1) << mbytesPerSec << "Mb/sec";
+                oss << " - " << std::fixed << std::setprecision(1) << mbytesPerSec << "MB/sec";
             } else {
-                oss << " - " << kbytesPerSec << " Kb/sec";
+                oss << " - " << kbytesPerSec << " KB/sec";
             }
         }
         fgSetString("/sim/startup/splash-progress-spinner", oss.str());
