@@ -26,6 +26,7 @@
 #endif
 
 #include "viewmgr.hxx"
+#include "ViewPropertyEvaluator.hxx"
 
 #include <simgear/compiler.h>
 #include <simgear/scene/util/OsgMath.hxx>
@@ -38,9 +39,9 @@
 // Constructor
 FGViewMgr::FGViewMgr( void ) :
   inited(false),
-  config_list(fgGetNode("/sim", true)->getChildren("view")),
-  current(0)
+  config_list(fgGetNode("/sim", true)->getChildren("view"))
 {
+    current = fgGetInt("/sim/current-view/view-number");
 }
 
 // Destructor
@@ -62,7 +63,7 @@ FGViewMgr::init ()
     SGPropertyNode *n = config_list[i];
     SGPropertyNode *config = n->getChild("config", 0, true);
 
-      flightgear::View* v = flightgear::View::createFromProperties(config);
+      flightgear::View* v = flightgear::View::createFromProperties(config, n->getIndex());
       if (v) {
           add_view(v);
       }
@@ -111,6 +112,7 @@ FGViewMgr::bind()
     _viewNumberProp = _tiedProperties.getRoot()->getNode("view-number");
     _viewNumberProp->setAttribute(SGPropertyNode::ARCHIVE, false);
     _viewNumberProp->setAttribute(SGPropertyNode::PRESERVE, true);
+    _viewNumberProp->setAttribute(SGPropertyNode::LISTENER_SAFE, true);
 }
 
 
@@ -124,6 +126,8 @@ FGViewMgr::unbind ()
 
   _tiedProperties.Untie();
     _viewNumberProp.clear();
+    
+    ViewPropertyEvaluator::clear();
 }
 
 void
@@ -251,9 +255,9 @@ FGViewMgr::setView (int newview)
     if (get_current_view()) {
         get_current_view()->bind();
     }
-
-    // force an update now, to avoid returning bogus data.
-    // real fix would to be make all the accessors use the dirty mechanism
-    // on FGViewer, so update() is a no-op.
-    update(0.0);
 }
+
+
+// Register the subsystem.
+SGSubsystemMgr::Registrant<FGViewMgr> registrantFGViewMgr(
+    SGSubsystemMgr::DISPLAY);
